@@ -6,35 +6,38 @@
 #include <string.h>
 #include <assert.h>
 
-#define NOS 12
+#define NOS 13
 static char *osnam[NOS] =
    {"UNKNOWN", "Linux", "SunOS", "SunOS4", "OSF1", "IRIX", "AIX",
-    "Win9x", "WinNT", "HPUX", "FreeBSD", "OSX"};
+    "Win9x", "WinNT", "WinSFU", "HPUX", "FreeBSD", "OSX"};
 enum OSTYPE {OSOther=0, OSLinux, OSSunOS, OSSunOS4, OSOSF1, OSIRIX, OSAIX,
-             OSWin9x, OSWinNT, OSHPUX, OSFreeBSD, OSOSX};
-#define OSIsWin(OS_) (((OS_) == OSWinNT) || ((OS_) == OSWin9x))
+             OSWin9x, OSWinNT, OSWinSFU, OSHPUX, OSFreeBSD, OSOSX};
+#define OSIsWin(OS_) ( ((OS_) == OSWinNT) || ((OS_) == OSWin9x) || \
+                       ((OS_) == OSWinSFU) )
 
-enum ARCHFAM {AFOther=0, AFPPC, AFSPARC, AFALPHA, AFX86, AFIA64, AFMIPS};
+enum ARCHFAM {AFOther=0, AFPPC, AFSPARC, AFALPHA, AFX86, AFIA64, AFMIPS, AFARM};
 
-#define NMACH 37
+#define NMACH 39
 static char *machnam[NMACH] =
    {"UNKNOWN", "POWER3", "POWER4", "POWER5", "PPCG4", "PPCG5",
     "POWER6", "POWER7",
     "P5", "P5MMX", "PPRO", "PII", "PIII", "PM", "CoreSolo",
-    "CoreDuo", "Core2Solo", "Core2", "Corei1", "Corei2", "P4", "P4E",
+    "CoreDuo", "Core2Solo", "Core2", "Corei1", "Corei2", "Atom", "P4", "P4E",
     "Efficeon", "K7", "HAMMER", "AMD64K10h", "UNKNOWNx86",
     "IA64Itan", "IA64Itan2",
     "USI", "USII", "USIII", "USIV", "UST2", "UnknownUS",
-    "MIPSR1xK", "MIPSICE9"};
+    "MIPSR1xK", "MIPSICE9", "ARMv7"};
 enum MACHTYPE {MACHOther, IbmPwr3, IbmPwr4, IbmPwr5, PPCG4, PPCG5,
                IbmPwr6, IbmPwr7,
                IntP5, IntP5MMX, IntPPRO, IntPII, IntPIII, IntPM, IntCoreS,
                IntCoreDuo, IntCore2Solo, IntCore2, IntCorei1, IntCorei2,
-               IntP4, IntP4E, TMEff, AmdAthlon, AmdHammer, Amd64K10h, x86X,
+               IntAtom, IntP4, IntP4E, TMEff,
+               AmdAthlon, AmdHammer, Amd64K10h, x86X,
                IA64Itan, IA64Itan2,
                SunUSI, SunUSII, SunUSIII, SunUSIV, SunUST2, SunUSX,
                MIPSR1xK, /* includes R10K, R12K, R14K, R16K */
-               MIPSICE9   /* SiCortex ICE9 -- like MIPS5K */
+               MIPSICE9,  /* SiCortex ICE9 -- like MIPS5K */
+               ARMv7      /* includes Cortex A8, A9 */
                };
 #define MachIsX86(mach_) \
    ( (mach_) >= IntP5 && (mach_) <= x86X )
@@ -51,6 +54,9 @@ enum MACHTYPE {MACHOther, IbmPwr3, IbmPwr4, IbmPwr5, PPCG4, PPCG5,
 #endif
 #define MachIsPPC(mach_) \
    ( (mach_) >= PPCG4 && (mach_) <= PPCG5 )
+#define MachIsARM(mach_) \
+   ( (mach_) == ARMv7 )
+
 
 static char *f2c_namestr[5] = {"UNKNOWN","Add_", "Add__", "NoChange", "UpCase"};
 static char *f2c_intstr[5] =
@@ -63,19 +69,22 @@ enum F2CNAME {f2c_NamErr=0, f2c_Add_, f2c_Add__, f2c_NoChange, f2c_UpCase};
 enum F2CINT {f2c_IntErr=0, FintCint, FintClong, FintClonglong, FintCshort};
 enum F2CSTRING {f2c_StrErr=0, fstrSun, fstrCray, fstrStructVal, fstrStructPtr};
 
-#define NISA 6
+#define NISA 10
 static char *ISAXNAM[NISA] =
-   {"", "AltiVec", "SSE3", "SSE2", "SSE1", "3DNow"};
-enum ISAEXT {ISA_None=0, ISA_AV, ISA_SSE3, ISA_SSE2, ISA_SSE1, ISA_3DNow};
+   {"", "VSX", "AltiVec", "AVXMAC", "AVX", "SSE3", "SSE2", "SSE1", "3DNow",
+    "NEON"};
+enum ISAEXT
+   {ISA_None=0, ISA_VSX, ISA_AV, ISA_AVXMAC, ISA_AVX,
+    ISA_SSE3, ISA_SSE2, ISA_SSE1, ISA_3DNow,
+    ISA_NEON};
 
-#define NASMD 7
+#define NASMD 8
 enum ASMDIA
    {ASM_None=0, gas_x86_32, gas_x86_64, gas_sparc, gas_ppc, gas_parisc,
-    gas_mips};
+    gas_mips, gas_arm};
 static char *ASMNAM[NASMD] =
    {"",     "GAS_x8632", "GAS_x8664", "GAS_SPARC", "GAS_PPC", "GAS_PARISC",
-    "GAS_MIPS"};
-
+    "GAS_MIPS", "GAS_ARM"};
 
 /*
  * Used for archinfo probes (can pack in bitfield)
