@@ -37,10 +37,19 @@ void PrintUsage(char *name, int iarg, char *arg)
            "      -Si bozol1 <0/1> : supress/enable bozo L1 defaults\n");
    fprintf(stderr,
            "      -Si archdef <1/0> : enable/supress arch default use\n");
+   fprintf(stderr,
+           "      -Si latune <1/0> : do/don't tune F77 LAPACK routines\n");
       fprintf(stderr,
         "      -Si nof77 <0/1> : Have/don't have fortran compiler\n");
+      fprintf(stderr,
+        "      -Si omp <0/1> : don'tuse/use OpenMP for threading\n");
+      fprintf(stderr,
+"      -Si antthr <0/1/2> : nobuild/build/use Antoine's code for threading\n");
+      fprintf(stderr,
+              "      -Si lapackref <0/1>: Netlib lapack is not/is unpacked\n");
+      fprintf(stderr, "                           to $BLDdir/src/lapack/ref\n");
    fprintf(stderr,
-        "      -Ss kern <path to comp> : use comp for all kernel compilers\n");
+        "      -Ss kern <path/to/comp> : use comp for all kernel compilers\n");
    fprintf(stderr,
         "      -Ss pmake <parallel make invocation (eg '$(MAKE) -j 4')>\n");
    fprintf(stderr,
@@ -63,6 +72,8 @@ void GetFlags(int nargs,                /* nargs as passed into main */
               int *mhz,                /* Clock rate in Mhz */
               int *ptrbits             /* # of bits in ptr: 32/64 */,
               int *nthreads,           /* # of threads */
+              int *omp,                /* Build OpenMP version of threading? */
+              int *AntThr,             /* Build Antoine's threads? */
               char **comps,
               char **gccflags,        /* append flags for user-contrib gcc */
               char **outfile,
@@ -70,7 +81,9 @@ void GetFlags(int nargs,                /* nargs as passed into main */
               char **bindir,          /* path to top of binary directory */
               int *bozol1,            /* Use untuned L1 */
               int *UseArchDef,        /* Use arch defaults */
+              int *latune,            /* Tune LAPACK? */
               int *NoF77,
+              int *lapackref,
               char **f2cdefs,         /* F77-to-C interface defines */
               char **ecdefs,          /* extra cpp defines to add to CDEFS */
               char **pmake,           /* parallel make command */
@@ -91,7 +104,7 @@ void GetFlags(int nargs,                /* nargs as passed into main */
    *verb = 0;
    *srcdir = *bindir = NULL;
     *bozol1 = 0;
-    *UseArchDef = 1;
+    *latune = *UseArchDef = 1;
     *flapack = NULL;
     *f77lib = NULL;
     *smaflags = *dmaflags = NULL;
@@ -113,6 +126,7 @@ void GetFlags(int nargs,                /* nargs as passed into main */
    *verb = 0;
    *NoF77 = 0;
    *nthreads = -1;
+   *omp = *AntThr = 0;
    *pmake = NULL;
    for (i=1; i < nargs; i++)
    {
@@ -187,6 +201,14 @@ void GetFlags(int nargs,                /* nargs as passed into main */
             *UseArchDef = k;
          else if (!strcmp(sp0, "bozol1"))
             *bozol1 = k;
+         else if (!strcmp(sp0, "latune"))
+            *latune = k;
+         else if (!strcmp(sp0, "omp"))
+            *omp = k;
+         else if (!strcmp(sp0, "antthr"))
+            *AntThr = k;
+         else if (!strcmp(sp0, "lapackref"))
+            *lapackref = k;
          else if (!strcmp(sp0, "nof77"))
             *NoF77 = k;
          else if (!strcmp(sp0, "kern"))
@@ -376,7 +398,7 @@ char *GetPtrbitsFlag(enum OSTYPE OS, enum MACHTYPE arch, int ptrbits,
    return(sp);
 }
 
-main(int nargs, char **args)
+int main(int nargs, char **args)
 {
    enum OSTYPE OS;
    enum MACHTYPE mach;
@@ -386,8 +408,10 @@ main(int nargs, char **args)
    int delay=0;  /* change this to come from "special" ints in GetFlags */
    int THREADS=0;
    int Use3DNow=0;  /* this needs to come from getflags */
-   int ncpu;
-   int USEDEFL1, USEARCHDEF;
+   int ncpu, omp, AntThr, lapackref;
+   int USEDEFL1, USEARCHDEF, LATUNE;
+   #define NPREC 4
+   char pres[NPREC] = {'s', 'd', 'c', 'z'};
    char *targ, *sp, *pmake, *flapack;
    char *comps[3*NCOMP], *comp, *flags, *srcdir, *blddir, *f2cdefs, *cdefs;
    char *outfile, *smaflags, *dmaflags, *f77lib, *gccflags, *goodgcc;
@@ -396,9 +420,10 @@ main(int nargs, char **args)
    char *adnames[NARDEF] = {"sKERNDEF", "dKERNDEF", "sMMDEF", "dMMDEF"};
 
    GetFlags(nargs, args, &verb, &OS, (enum ASMDIA*) &asmb, &vecexts, &mach,
-            &mhz, &ptrbits, &ncpu, comps, &gccflags, &outfile, &srcdir,
-            &blddir, &USEDEFL1, &USEARCHDEF, &nof77, &f2cdefs, &cdefs, &pmake,
-            &flapack, &smaflags, &dmaflags, &f77lib, &l2size, &targ);
+            &mhz, &ptrbits, &ncpu, &omp, &AntThr, comps, &gccflags, &outfile,
+            &srcdir, &blddir, &USEDEFL1, &USEARCHDEF, &LATUNE, &nof77,
+            &lapackref, &f2cdefs, &cdefs, &pmake, &flapack,
+            &smaflags, &dmaflags, &f77lib, &l2size, &targ);
    if (ncpu > 1) THREADS = 1;
    if (!outfile)
       fpout = stdout;
@@ -471,7 +496,7 @@ main(int nargs, char **args)
          ISAX = i;
 
    fprintf(fpout, "#  ----------------------------\n");
-   fprintf(fpout, "#  Make.inc for ATLAS3.8.4\n");
+   fprintf(fpout, "#  Make.inc for ATLAS3.9.51\n");
    fprintf(fpout, "#  ----------------------------\n\n");
 
    fprintf(fpout, "#  ----------------------------------\n");
@@ -514,6 +539,7 @@ main(int nargs, char **args)
    fprintf(fpout, "   R1Tdir = $(BLDdir)/tune/blas/ger\n");
    fprintf(fpout, "   L1Tdir = $(BLDdir)/tune/blas/level1\n");
    fprintf(fpout, "   L3Tdir = $(BLDdir)/tune/blas/level3\n");
+   fprintf(fpout, "   FLAdir = $(BLDdir)/src/lapack/reference\n");
    fprintf(fpout, "\n");
 
    fprintf(fpout,
@@ -531,12 +557,13 @@ main(int nargs, char **args)
    fprintf(fpout, "   ATLASlib = $(LIBdir)/libatlas.a\n");
    fprintf(fpout, "   CBLASlib = $(LIBdir)/libcblas.a\n");
    fprintf(fpout, "   F77BLASlib = $(LIBdir)/libf77blas.a\n");
+   fprintf(fpout, "   LAPACKlib = $(LIBdir)/liblapack.a\n");
    if (THREADS)
    {
       fprintf(fpout, "   PTCBLASlib = $(LIBdir)/libptcblas.a\n");
       fprintf(fpout, "   PTF77BLASlib = $(LIBdir)/libptf77blas.a\n");
+      fprintf(fpout, "   PTLAPACKlib = $(LIBdir)/libptlapack.a\n");
    }
-   fprintf(fpout, "   LAPACKlib = $(LIBdir)/liblapack.a\n");
    fprintf(fpout, "   TESTlib = $(LIBdir)/libtstatlas.a\n\n");
 
    fprintf(fpout, "#  -------------------------------------------\n");
@@ -583,11 +610,9 @@ main(int nargs, char **args)
 /*
  * Need up update handling of apple vs. gnu gcc for altivec
  */
-#if 0
-   if (ISAX == ISA_AV && strstr(CC, "gcc") && strstr(MCC, "gcc") &&
-       !GccIsApple(CC))
-      fprintf(fpout, " -DATL_%s -DATL_AVgcc", ISAXNAM[ISAX]);
-#endif
+   if ((ISAX == ISA_AV || ISAX == ISA_VSX) && CompIsGcc(comps[DMC_]) &&
+        !CompIsAppleGcc(comps[DMC_]))
+      fprintf(fpout, " -DATL_AVgcc");
    fprintf(fpout, "\n\n");
 
    fprintf(fpout,
@@ -604,14 +629,18 @@ main(int nargs, char **args)
 
    sprintf(ln, "%s/CONFIG/src/CompMake.txt", srcdir);
    DisplayFile(ln, fpout, 0);
+   fprintf(fpout, "   NPROC=%d\n", ncpu);
    fprintf(fpout, "   CDEFS = $(L2SIZE) $(INCLUDES) $(F2CDEFS) $(ARCHDEFS)");
    if (cdefs) fprintf(fpout, " %s", cdefs);
    if (THREADS)
    {
-      fprintf(fpout, " -DATL_NCPU=%d", ncpu);
+      fprintf(fpout, " -DATL_NCPU=$(NPROC)");
       if (OS == OSFreeBSD) fprintf(fpout, " -D_THREAD_SAFE -D_REENTRANT");
       if (OS == OSAIX) fprintf(fpout, " -DIBM_PT_ERROR");
       if (OS == OSIRIX) fprintf(fpout, " -D_POSIX_C_SOURCE=199506L");
+      if (AntThr) fprintf(fpout, " -DATL_TRUST_ANTPT");
+      else if (omp) fprintf(fpout, " -DATL_OMP_THREADS");
+      if (AntThr > 1) fprintf(fpout, " -DATL_ANTOINE_THREADS");
    }
    if (delay) fprintf(fpout, " -DATL_FOPENDELAY");
    fprintf(fpout, "\n\n");
@@ -627,6 +656,7 @@ main(int nargs, char **args)
          fprintf(fpout, "   %sFLAGS = %s\n",
                  COMPNAME[i], comps[NCOMP+i]);
    }
+   fprintf(fpout, "   F77NOOPT = -O0   # turn off optimization\n");
    fprintf(fpout, "   SMAFLAGS =");
    if (smaflags)
       fprintf(fpout, " %s", smaflags);
@@ -649,43 +679,63 @@ main(int nargs, char **args)
           comps[i][j-2] == 'c' && comps[i][j-1] == 'c')
          break;
    }
-   goodgcc = (j < NCOMP) ? comps[j] : "gcc";
+   goodgcc = (i < NCOMP) ? comps[i] : "gcc";
    fprintf(fpout, "   GOODGCC = %s", goodgcc);
    if (gccflags)
       fprintf(fpout, " %s", gccflags);
    GetGccVers(goodgcc, &i, &j, &k, &k);
    if (OS == OSOSX && j > 3)  /* need bullshit apple annoyance flag */
       fprintf(fpout, " -force_cpusubtype_ALL");
+   if (OSIsWin(OS) && ptrbits != 64)  /* stop gcc breaking ABI */
+      fprintf(fpout, " -mstackrealign");
    sp = GetPtrbitsFlag(OS, mach, ptrbits, goodgcc);
    if (strlen(sp) > 0)
        fprintf(fpout, " %s", sp);
+   #ifdef ATL_DYLIBS
+      if (!OSIsWin(OS))
+         fprintf(fpout, " -fPIC");
+   #endif
    fprintf(fpout, "\n");
 
    fprintf(fpout, "   LDFLAGS =");
    if (MachIsX86(mach))
    {
-      if (ptrbits == 32)
-         fprintf(fpout, " -melf_i386");
-      else if (ptrbits == 64)
-         fprintf(fpout, " -melf_x86_64");
-      if (OS == OSFreeBSD)
-         fprintf(fpout, "_fbsd");
+      if (OSIsWin(OS))
+         fprintf(fpout, " -mi386pe");
+      else
+      {
+         if (ptrbits == 32)
+            fprintf(fpout, " -melf_i386");
+         else if (ptrbits == 64)
+            fprintf(fpout, " -melf_x86_64");
+         if (OS == OSFreeBSD)
+            fprintf(fpout, "_fbsd");
+      }
    }
    fprintf(fpout, "\n   F77SYSLIB = %s\n", f77lib ? f77lib : "");
    fprintf(fpout, "   BC = $(ICC)\n");
    fprintf(fpout, "   NCFLAGS = $(ICCFLAGS)\n");
    fprintf(fpout, "\n   CLINKER = $(ICC)\n   CLINKFLAGS = $(ICCFLAGS)\n");
-   fprintf(fpout, "   FLINKER = $(F77)\n   FLINKFLAGS = $(F77FLAGS)\n");
+   fprintf(fpout, "   FLINKER = $(F77)\n");
+   if (strstr(comps[F77_], "mgwgfortran"))
+      fprintf(fpout, "   FLINKFLAGS = $(F77FLAGS) -static\n");
+   else
+      fprintf(fpout, "   FLINKFLAGS = $(F77FLAGS)\n");
    fprintf(fpout, "   FCLINKFLAGS = $(FLINKFLAGS)");
    if (strstr(comps[F77_], "ifort") && !OSIsWin(OS))
       fprintf(fpout, " -nofor_main");
-   fprintf(fpout, "\n   ARCHIVER = ar\n");
+   if (OS == OSWinNT && ptrbits == 64)
+      fprintf(fpout, "\n   ARCHIVER = $(BLDdir)/mgwar\n");
+   else
+      fprintf(fpout, "\n   ARCHIVER = ar\n");
    fprintf(fpout, "   ARFLAGS  = r\n");
 /*
  * JF Mertens says that even x86 OS X still need ranlib for safety
  */
-   if (OS == OSOSX /* && MachIsPPC(mach) */)
+   if (OS == OSOSX)
       fprintf(fpout, "   RANLIB   = ranlib\n\n");
+   else if (OS == OSWinNT && ptrbits == 64)
+      fprintf(fpout, "   RANLIB   = $(BLDdir)/mgwranlib\n\n");
    else
       fprintf(fpout, "   RANLIB   = echo\n\n");
 
@@ -702,24 +752,30 @@ main(int nargs, char **args)
    fprintf(fpout, "#  ------------------------------------\n");
    fprintf(fpout, "#  Reference and system libraries\n");
    fprintf(fpout, "#  ------------------------------------\n");
-   fprintf(fpout, "   BLASlib = \n");
-   fprintf(fpout, "   FBLASlib = \n");
+   fprintf(fpout, "   FBLASlib = $(LIBdir)/libf77refblas.a\n");
    fprintf(fpout, "   FLAPACKlib = ");
    if (flapack) fprintf(fpout, "%s", flapack);
+   else if (lapackref) fprintf(fpout, "$(FLAdir)/lapack_$(ARCH).a");
    fprintf(fpout, "\n");
-   fprintf(fpout, "   SLAPACKlib = \n");
+   fprintf(fpout, "   SBLASlib = $(FBLASlib)  # should be serial sysblas\n");
+   fprintf(fpout, "   BLASlib = $(FBLASlib)   # should be parallel sysblas\n");
+   fprintf(fpout, "   SLAPACKlib =   # set to parallel system lapack\n");
+   fprintf(fpout, "   SSLAPACKlib =  # set to serial system lapack\n");
    if (THREADS)
+   {
       fprintf(fpout, "   LIBS = -lpthread");
+      if (OSIsWin(OS) && ptrbits == 32)
+         fprintf(fpout, " -lkernel32");
+   }
    else
       fprintf(fpout, "   LIBS =");
-   if (!OSIsWin(OS))
-      fprintf(fpout, " -lm");
+   fprintf(fpout, " -lm");
    fprintf(fpout, "\n\n");
 
    fprintf(fpout,
    "#  --------------------------------------------------------------------\n");
    fprintf(fpout,
-   "#  Compiler names for architectural defaults and flags to atlas_build\n");
+   "#  Compiler names for architectural defaults and flags to atlas_install\n");
    fprintf(fpout,
    "#  --------------------------------------------------------------------\n");
    for (i=0; i < NCOMP; i++)
@@ -753,8 +809,22 @@ main(int nargs, char **args)
       fprintf(fpout, "%s\n", sp);
       free(sp);
    }
-   fprintf(fpout, "   INSTFLAGS = -1 %d -a %d\n\n", USEDEFL1, USEARCHDEF);
+   fprintf(fpout, "   INSTFLAGS = -1 %d -a %d -l %d\n\n",
+           USEDEFL1, USEARCHDEF, LATUNE);
 
+fprintf(fpout,
+   "#  -------------------------------------------------------------------\n");
+fprintf(fpout,
+   "#  Dependence info for building optional external threading interfaces\n");
+fprintf(fpout,
+   "#  -------------------------------------------------------------------\n");
+   for (i=0; i < NPREC; i++)
+   {
+      fprintf(fpout, "   %cextthr =", pres[i]);
+      if (AntThr)
+         fprintf(fpout, " %cpt", pres[i]);
+      fprintf(fpout, "\n");
+   }
    fprintf(fpout, "#  ---------------------------------------\n");
    fprintf(fpout, "#  Generic targets needed by all makefiles\n");
    fprintf(fpout, "#  ---------------------------------------\n");
@@ -762,10 +832,10 @@ main(int nargs, char **args)
    if (delay)
    {
       fprintf(fpout, "   waitfile = wfdefault\n");
-      fprintf(fpout, "waitfile:\n\tcd $(BINdir) ; make xatlas_waitfile\n");
+      fprintf(fpout, "waitfile:\n\tcd $(BINdir) ; $(MAKE) xatlas_waitfile\n");
       fprintf(fpout, "\t$(ATLFWAIT) -s %d -f $(waitfile)\n", delay);
    }
    else fprintf(fpout, "waitfile:\n");
    if (fpout != stdout && fpout != stderr) fclose(fpout);
-   exit(0);
+   return(0);
 }
