@@ -1,5 +1,5 @@
 /*
- *             Automatically Tuned Linear Algebra Software v3.8.4
+ *             Automatically Tuned Linear Algebra Software v3.9.51
  *                    (C) Copyright 1998 R. Clint Whaley
  *
  * Redistribution and use in source and binary forms, with or without
@@ -30,6 +30,7 @@
 #include "atlconf.h"
 
 #include <time.h>
+#include <ctype.h>
 void GetDate(int *month, int *day, int *year, int *hour, int *min)
 {
    time_t tv;
@@ -52,7 +53,7 @@ long GetInt(FILE *fpin, long Default, char *spc, char *expstr)
 {
    char str[64];
    long iin;
-   if (expstr) fprintf(stdout, "%sEnter %s [%d]: ", spc, expstr, Default);
+   if (expstr) fprintf(stdout, "%sEnter %s [%ld]: ", spc, expstr, Default);
    if (fgets(str, 64, fpin) == NULL) return(Default);
    if (sscanf(str, " %ld ", &iin) != 1) return(Default);
    return(iin);
@@ -68,13 +69,13 @@ long GetIntRange(long Default, long Min, long Max, char *spc, char *expstr)
       if (i > Max)
       {
          keepOn = 1;
-         fprintf(stderr, "\n%d larger than max value of %d.  Try again.\n\n",
+         fprintf(stderr, "\n%ld larger than max value of %ld.  Try again.\n\n",
                  i, Max);
       }
       else if (i < Min)
       {
          keepOn = 1;
-         fprintf(stderr, "\n%d smaller than min value of %d.  Try again.\n\n",
+         fprintf(stderr, "\n%ld smaller than min value of %ld.  Try again.\n\n",
                  i, Min);
       }
       else keepOn = 0;
@@ -90,7 +91,7 @@ long GetIntVer(long Default, long Min, long Max, char *spc, char *expstr)
    do
    {
       i = GetIntRange(Default, Min, Max, spc, expstr);
-      fprintf(stdout, "%s   You entered: %d\n", spc, i);
+      fprintf(stdout, "%s   You entered: %ld\n", spc, i);
       j = GetIntRange(0, 0, 1, spc, "1 to reenter, 0 accepts");
    }
    while(j);
@@ -480,6 +481,7 @@ enum ARCHFAM ProbeArchFam(char *targ)
       else if (strstr(res, "alpha")) fam = AFALPHA;
       else if (strstr(res, "ia64")) fam = AFIA64;
       else if (strstr(res, "mips")) fam = AFMIPS;
+      else if (strstr(res, "arm")) fam = AFARM;
       else if ( strstr(res, "i686") || strstr(res, "i586") ||
                 strstr(res, "i486") || strstr(res, "i386") ||
                 strstr(res, "x86") || strstr(res, "x86_64") ) fam = AFX86;
@@ -501,6 +503,7 @@ enum ARCHFAM ProbeArchFam(char *targ)
                    strstr(res, "i486") || strstr(res, "i386") ||
                    strstr(res, "x86_64") ) fam = AFX86;
          else if (strstr(res, "mips")) fam = AFMIPS;
+         else if (strstr(res, "arm")) fam = AFARM;
       }
    }
    return(fam);
@@ -726,6 +729,19 @@ int CompIsMIPSpro(char *comp)
    return(0);
 }
 
+int CompIsPathScale(char *comp)
+{
+   char cmnd[1024], res[1024];
+
+   sprintf(cmnd, "%s -v", comp);
+   if (CmndOneLine(NULL, cmnd, res) == 0)
+   {
+      if (strstr(res, "PathScale"))
+         return(1);
+   }
+   return(0);
+}
+
 int CompIsSunWorkshop(char *comp)
 /*
  * RETURNS: 1 if compiler is Sun WorkShop compiler, 0 otherwise
@@ -768,6 +784,29 @@ char *NewStringCopy(char *old)
    strcpy(new, old);
    return(new);
 }
+char *NewAppendedString0(char *old, char *app)
+/*
+ * RETURNS: string holding : old + app
+ * NOTE: frees old string after copy
+ */
+{
+   char *new;
+   if (!old)
+   {
+      new = malloc(sizeof(char)*(strlen(app)+1));
+      assert(new);
+      strcpy(new, app);
+   }
+   else
+   {
+      new = malloc(sizeof(char)*(strlen(old) + strlen(app)+1));
+      assert(new);
+      strcpy(new, old);
+      strcat(new, app);
+      free(old);
+   }
+   return(new);
+}
 char *NewAppendedString(char *old, char *app)
 /*
  * RETURNS: string holding : old + " " + app
