@@ -98,6 +98,16 @@ enum MACHTYPE ProbeArch()
             mach = MIPSR1xK;
       }
       break;
+   case AFARM:
+      if ( !CmndOneLine(NULL, "fgrep 'Processor' /proc/cpuinfo", res) )
+      {
+         if (strstr(res, "ARMv7") || strstr(res,"v7l")) mach = ARMv7;
+      }
+      else if ( !CmndOneLine(NULL, "fgrep cpu /proc/cpuinfo", res) )
+      {
+         if (strstr(res, "ARMv7") || strstr(res,"v7l")) mach = ARMv7;
+      }
+      break;
    case AFIA64:
       res[0] = '\0';
       ierr = CmndOneLine(NULL, "fgrep 'Itanium' /proc/cpuinfo", res);
@@ -135,6 +145,8 @@ enum MACHTYPE ProbeArch()
                }
             }
          }
+         else if (strstr(res, "Atom"))
+            mach = IntAtom;
          else if (strstr(res, "Core"))
          {
             if (strstr(res, "i7"))
