@@ -161,6 +161,31 @@ char *ProbeComp(int verb, char *targarg, enum OSTYPE OS, enum MACHTYPE arch,
          }
       }
    }
+/*
+ * Add the flag to build shared objects, if we know what it is
+ */
+#ifdef ATL_DYLIBS
+   for (i=0; i < NCOMP; i++)
+   {
+      if (i != XCC_)
+      {
+         if (CompIsGcc(comps[i]) || CompIsAppleGcc(comps[i]) ||
+             CompIsPathScale(comps[i]))
+         {
+            if (!OSIsWin(OS))  /* windows/gcc bitches about -fPIC flag */
+               comps[i+NCOMP] = NewAppendedString(comps[i+NCOMP],"-fPIC");
+         }
+         else if (CompIsMIPSpro(comps[i]))
+            comps[i+NCOMP] = NewAppendedString(comps[i+NCOMP],"-KPIC");
+         else if (CompIsSunWorkshop(comps[i]))
+            comps[i+NCOMP] = NewAppendedString(comps[i+NCOMP],"-KPIC");
+#if 0 /* I don't think a special flag is needed for shared obj wt xlc */
+         else if (CompIsIBMXL(comps[i]))
+            comps[i+NCOMP] = NewAppendedString(comps[i+NCOMP],"????");
+#endif
+      }
+   }
+#endif
 /*
  * Only f77 and xcc allowed to be NULL
  */
@@ -284,7 +309,7 @@ char *ProbeF77LIB(int verb, char *targarg, enum OSTYPE OS,
             if (verb > 1) fprintf(stderr, "SUCCESS!\n");
             F77LIBdir = NewStringCopy(sp-2);
             F77LIBdir = NewAppendedString(F77LIBdir, "-l");
-            F77LIBdir = NewAppendedString(F77LIBdir, f77lib);
+            F77LIBdir = NewAppendedString0(F77LIBdir, f77lib);
             if (OSIsWin(OS))
                F77LIBdir = NewAppendedString(F77LIBdir, "-lgcc");
             if (verb)
@@ -354,6 +379,8 @@ int ProbeMhz(int verb, char *targarg, enum OSTYPE OS, enum ASMDIA asmb)
    char ln[1024];
 
    sprintf(ln, "%s -O %d -s %d -m", targarg, OS, asmb);
+   if (verb > 2)
+      printf("Mhz Probe = '%s'\n", ln);
    iret = GetIntProbe(verb, ln, "arch", "CPU MHZ", 16384);
    printf("\nClock rate configured as %dMhz\n", iret);
    return(iret);
@@ -443,9 +470,11 @@ char *ProbePmake(int verb, enum OSTYPE OS, int ncpu)
 }
 
 void SpewItForth(int verb, enum OSTYPE OS, enum MACHTYPE arch, int mhz,
-                 enum ASMDIA asmb, int vecexts, int ptrbits, int ncpu,
+                 enum ASMDIA asmb, int vecexts, int ptrbits,
+                 int ncpu, int omp, int AntThr,
                  int l2size, char *srcdir, char *bindir, int bozol1,
-                 int archdef, int nof77, char **comps, char *gccflags,
+                 int archdef, int latune,  int nof77, char **comps,
+                 char *gccflags,
                  char *f2cdefs, char *cdefs, char *pmake, char *flapack,
                  char *smaflags, char *dmaflags, char *f77libs)
 /*
@@ -459,10 +488,10 @@ void SpewItForth(int verb, enum OSTYPE OS, enum MACHTYPE arch, int mhz,
  * Translate compiler/flag array to xspew flags
  */
    Comps2Flags(comps, compsflags);
-   i = sprintf(ln, "./xspew -v %d -O %d -A %d -m %d -s %d -V %d -b %d -t %d -f %d -d s '%s' -d b '%s' -D c '%s' -D f '%s' %s -Si archdef %d -Si bozol1 %d -Si nof77 %d -o Make.inc",
+   i = sprintf(ln, "./xspew -v %d -O %d -A %d -m %d -s %d -V %d -b %d -t %d -f %d -d s '%s' -d b '%s' -D c '%s' -D f '%s' %s -Si archdef %d -Si bozol1 %d -Si latune %d -Si nof77 %d -o Make.inc",
                verb, OS, arch, mhz, asmb, vecexts, ptrbits, ncpu, l2size,
                srcdir, bindir, cdefs ? cdefs:"", f2cdefs? f2cdefs : "",
-               compsflags, archdef, bozol1, nof77);
+               compsflags, archdef, bozol1, latune, nof77);
    if (pmake)
       i += sprintf(ln+i, " -Ss pmake '%s'", pmake);
    if (flapack)
@@ -473,6 +502,10 @@ void SpewItForth(int verb, enum OSTYPE OS, enum MACHTYPE arch, int mhz,
       i += sprintf(ln+i, " -Ss dmaflags '%s'", dmaflags);
    if (f77libs)
       i += sprintf(ln+i, " -Ss f77lib '%s'", f77libs);
+   if (omp)
+      i += sprintf(ln+i, " -Si omp %d", omp);
+   if (AntThr)
+      i += sprintf(ln+i, " -Si antthr %d", AntThr);
    if (gccflags)
       i += sprintf(ln+i, " -Fa gc '%s'", gccflags);
    if (verb > 1)
@@ -517,6 +550,8 @@ void PrintUsage(char *name, int iarg, char *arg)
            "      -Si bozol1 <0/1> : supress/enable bozo L1 defaults\n");
    fprintf(stderr,
            "      -Si archdef <1/0> : enable/supress arch default use\n");
+   fprintf(stderr,
+           "      -Si latune <1/0> : do/don't tune F77 LAPACK routines\n");
       fprintf(stderr,
         "      -Si nof77 <0/1> : Have/don't have fortran compiler\n");
       fprintf(stderr,
@@ -525,8 +560,15 @@ void PrintUsage(char *name, int iarg, char *arg)
         "                           (Windows compiler/cygwin install only)\n");
       fprintf(stderr,
         "      -Si cputhrchk <0/1> : Ignore/heed CPU throttle probe\n");
+      fprintf(stderr,
+        "      -Si omp <0/1> : don'tuse/use OpenMP for threading\n");
+      fprintf(stderr,
+"      -Si antthr <0/1/2> : nobuild/build/use Antoine's code for threading\n");
+      fprintf(stderr,
+              "      -Si lapackref <0/1>: Netlib lapack is not/is unpacked\n");
+      fprintf(stderr, "                           to $BLDdir/src/lapack/ref\n");
    fprintf(stderr,
-        "      -Ss kern <path to comp> : use comp for all kernel compilers\n");
+        "      -Ss kern <path/to/comp> : use comp for all kernel compilers\n");
    fprintf(stderr,
         "      -Ss pmake <parallel make invocation (eg '$(MAKE) -j 4')>\n");
    fprintf(stderr,
@@ -549,6 +591,8 @@ void GetFlags(int nargs,                /* nargs as passed into main */
               int *mhz,                /* Clock rate in Mhz */
               int *ptrbits             /* # of bits in ptr: 32/64 */,
               int *nthreads,           /* # of threads */
+              int *omp,                /* Build OpenMP version of threading? */
+              int *AntThr,             /* Build Antoine's threads? */
               char **comps,
               char **gccflags,        /* append flags for user-contrib gcc */
               char **outfile,
@@ -556,9 +600,11 @@ void GetFlags(int nargs,                /* nargs as passed into main */
               char **bindir,          /* path to top of binary directory */
               int *bozol1,            /* Use untuned L1 */
               int *UseArchDef,        /* Use arch defaults */
+              int *latune,            /* Tune LAPACK? */
               int *NoF77,
               int *NoCygwin,
               int *ThrChk,
+              int *lapackref,
               char **f2cdefs,         /* F77-to-C interface defines */
               char **ecdefs,          /* extra cpp defines to add to CDEFS */
               char **pmake,           /* parallel make command */
@@ -579,7 +625,7 @@ void GetFlags(int nargs,                /* nargs as passed into main */
    *verb = 0;
    *srcdir = *bindir = NULL;
     *bozol1 = 0;
-    *UseArchDef = 1;
+    *latune = *UseArchDef = 1;
     *flapack = NULL;
     *f77lib = NULL;
     *smaflags = *dmaflags = NULL;
@@ -603,6 +649,7 @@ void GetFlags(int nargs,                /* nargs as passed into main */
    *NoF77 = 0;
    *ThrChk = 1;
    *nthreads = -1;
+   *omp = *AntThr = 0;
    *pmake = NULL;
    for (i=1; i < nargs; i++)
    {
@@ -677,6 +724,14 @@ void GetFlags(int nargs,                /* nargs as passed into main */
             *UseArchDef = k;
          else if (!strcmp(sp0, "bozol1"))
             *bozol1 = k;
+         else if (!strcmp(sp0, "latune"))
+            *latune = k;
+         else if (!strcmp(sp0, "omp"))
+            *omp = k;
+         else if (!strcmp(sp0, "antthr"))
+            *AntThr = k;
+         else if (!strcmp(sp0, "lapackref"))
+            *lapackref = k;
          else if (!strcmp(sp0, "nof77"))
             *NoF77 = k;
          else if (!strcmp(sp0, "nocygwin"))
@@ -824,23 +879,24 @@ void GetFlags(int nargs,                /* nargs as passed into main */
       *ptrbits = 0;
 }
 
-main(int nargs, char **args)
+int main(int nargs, char **args)
 {
    enum OSTYPE OS;
    enum MACHTYPE mach;
    int i, verb, asmb, f2cname, f2cint, f2cstr, ncpu, nof77, nocygwin;
-   int thrchk, mhz;
+   int thrchk, mhz, omp, AntThr, lapackref;
    int j, k, h, vecexts;
-   int ptrbits, l2size, bozol1, archdef;
+   int ptrbits, l2size, bozol1, latune, archdef;
    char *targ, *f2cdefs, *cdefs, *srcdir, *bindir, *outfile, *sp;
    char targarg[256];
    char *comps[3*NCOMP], *gccflags;
    char *pmake, *flapack, *smaflags, *dmaflags, *f77libs;
 
    GetFlags(nargs, args, &verb, &OS, (enum ASMDIA*) &asmb, &vecexts, &mach,
-            &mhz, &ptrbits, &ncpu, comps, &gccflags, &outfile, &srcdir, &bindir,
-            &bozol1, &archdef, &nof77, &nocygwin, &thrchk, &f2cdefs, &cdefs,
-            &pmake, &flapack, &smaflags, &dmaflags, &f77libs, &l2size, &targ);
+            &mhz, &ptrbits, &ncpu, &omp, &AntThr, comps, &gccflags, &outfile,
+            &srcdir, &bindir, &bozol1, &archdef, &latune, &nof77, &nocygwin,
+            &thrchk, &lapackref, &f2cdefs, &cdefs, &pmake, &flapack,
+            &smaflags, &dmaflags, &f77libs, &l2size, &targ);
    if (targ)
       sprintf(targarg, "-T %s", targ);
    else
@@ -909,7 +965,8 @@ main(int nargs, char **args)
          else if (i == 3) /* reduce opt to avoid compiler hang */
             smaflags = "-O0";
       }
-      else if (OS == OSIRIX && CompIsMIPSpro(comps[SKC_]))
+      else if ((OS == OSIRIX && CompIsMIPSpro(comps[SKC_])) ||
+               CompIsPathScale(comps[SKC_]))
          smaflags = " -O2";
    }
    if (!dmaflags)
@@ -922,15 +979,17 @@ main(int nargs, char **args)
          else if (i == 3) /* reduce opt to avoid compiler hang */
             dmaflags = "-O0";
       }
-      else if (OS == OSIRIX && CompIsMIPSpro(comps[DKC_]))
+      else if (OS == OSIRIX && CompIsMIPSpro(comps[DKC_]) ||
+               CompIsPathScale(comps[DKC_]))
          dmaflags = " -O2";
    }
-   SpewItForth(verb, OS, mach, mhz, asmb, vecexts, ptrbits, ncpu, l2size,
-               srcdir, bindir, bozol1, archdef, nof77, comps, gccflags,
-               f2cdefs, cdefs, pmake, flapack, smaflags, dmaflags, f77libs);
+   SpewItForth(verb, OS, mach, mhz, asmb, vecexts, ptrbits, ncpu, omp, AntThr,
+               l2size, srcdir, bindir, bozol1, archdef, latune, nof77,
+               comps, gccflags, f2cdefs, cdefs, pmake, flapack,
+               smaflags, dmaflags, f77libs);
 /*
  * Cleanup directory, and exit
  */
    system("make confclean");
-   exit(0);
+   return(0);
 }
