@@ -1,5 +1,5 @@
 /*
- *             Automatically Tuned Linear Algebra Software v3.8.4
+ *             Automatically Tuned Linear Algebra Software v3.9.51
  *                    (C) Copyright 1998 R. Clint Whaley
  *
  * Redistribution and use in source and binary forms, with or without
@@ -34,6 +34,13 @@
 #include <ctype.h>
 #include <string.h>
 #include "atlas_fopen.h"
+#include "atlas_enum.h"
+#include "atlas_r1parse.h"
+#include "atlas_mvparse.h"
+#include "atlas_mmparse.h"
+#ifndef Mmax
+   #define Mmax(x, y) ( (x) > (y) ? (x) : (y) )
+#endif
 
 char *redir="2>&1 | ./xatlas_tee";
 char *fmake="make -f Makefile";
@@ -45,6 +52,7 @@ FILE *fpI, *fparch;
 #define Mlowcase(C) ( ((C) > 64 && (C) < 91) ? (C) | 32 : (C) )
 
 #include <time.h>
+#include <ctype.h>
 void GetDate(int *month, int *day, int *year, int *hour, int *min)
 {
    time_t tv;
@@ -67,7 +75,7 @@ long GetInt(FILE *fpin, long Default, char *spc, char *expstr)
 {
    char str[64];
    long iin;
-   if (expstr) fprintf(stdout, "%sEnter %s [%d]: ", spc, expstr, Default);
+   if (expstr) fprintf(stdout, "%sEnter %s [%ld]: ", spc, expstr, Default);
    if (fgets(str, 64, fpin) == NULL) return(Default);
    if (sscanf(str, " %ld ", &iin) != 1) return(Default);
    return(iin);
@@ -83,13 +91,13 @@ long GetIntRange(long Default, long Min, long Max, char *spc, char *expstr)
       if (i > Max)
       {
          keepOn = 1;
-         fprintf(stderr, "\n%d larger than max value of %d.  Try again.\n\n",
+         fprintf(stderr, "\n%ld larger than max value of %ld.  Try again.\n\n",
                  i, Max);
       }
       else if (i < Min)
       {
          keepOn = 1;
-         fprintf(stderr, "\n%d smaller than min value of %d.  Try again.\n\n",
+         fprintf(stderr, "\n%ld smaller than min value of %ld.  Try again.\n\n",
                  i, Min);
       }
       else keepOn = 0;
@@ -105,7 +113,7 @@ long GetIntVer(long Default, long Min, long Max, char *spc, char *expstr)
    do
    {
       i = GetIntRange(Default, Min, Max, spc, expstr);
-      fprintf(stdout, "%s   You entered: %d\n", spc, i);
+      fprintf(stdout, "%s   You entered: %ld\n", spc, i);
       j = GetIntRange(0, 0, 1, spc, "1 to reenter, 0 accepts");
    }
    while(j);
@@ -298,7 +306,7 @@ long long GetLastLong(char *ln)
 }
 
 void PrintBanner(char *fnam, int START, int sec, int subsec, int subsubsec);
-static void ATL_Cassert0(int cond, char *exp, char *logfile, int line)
+static void ATL_Cassert0(size_t cond, char *exp, char *logfile, int line)
 {
    FILE *fperr;
 
@@ -323,7 +331,7 @@ static void ATL_Cassert0(int cond, char *exp, char *logfile, int line)
    exit(-1);
 }
 #define ATL_Cassert(cond_, exp_, logfile_) \
-    ATL_Cassert0((int)(cond_), exp_, logfile_, __LINE__)
+    ATL_Cassert0((size_t)(cond_), exp_, logfile_, __LINE__)
 
 void GetInstLogFile(char *nam, int pre, int *muladd, int *pf, int *lat,
                     int *nb, int *mu, int *nu, int *ku, int *ForceFetch,
@@ -354,15 +362,99 @@ void PrintBanner(char *fnam, int START, int sec, int subsec, int subsubsec)
    GetDate(&month, &day, &year, &hour, &min);
    fprintf(fp, "\n%s%s%s", sep, sep, sep);
    if (START)
-      fprintf(fp, "*       BEGAN ATLAS3.8.4  INSTALL OF SECTION %1d-%1d-%1d ON %02d/%02d/%04d AT %02d:%02d     *\n",
+      fprintf(fp, "*       BEGAN ATLAS3.9.51 INSTALL OF SECTION %1d-%1d-%1d ON %02d/%02d/%04d AT %02d:%02d     *\n",
               sec, subsec, subsubsec, month, day, year, hour, min);
    else
-      fprintf(fp, "*      FINISHED ATLAS3.8.4  INSTALL OF SECTION %1d-%1d-%1d ON %02d/%02d/%04d AT %02d:%02d   *\n",
+      fprintf(fp, "*      FINISHED ATLAS3.9.51 INSTALL OF SECTION %1d-%1d-%1d ON %02d/%02d/%04d AT %02d:%02d   *\n",
                       sec, subsec, subsubsec, month, day, year, hour, min);
    fprintf(fp, "%s%s%s\n\n\n", sep, sep, sep);
    fclose(fp);
 }
 
+void PrintStartStop(FILE *fp0, FILE *fp1, int nspc, int START, int sec,
+                    int subsec, int subsubsec, char *stagename)
+{
+   int i, month, day, year, hour, min;
+   char ln[512];
+   FILE *fp;
+
+   GetDate(&month, &day, &year, &hour, &min);
+   for (i=0; i < nspc; i++)
+       ln[i] = ' ';
+   if (START)
+   {
+      if (stagename)
+         sprintf(ln+i, "BEGIN STAGE %d-%d-%d: %s at %02d:%02d\n",
+                 sec, subsec, subsubsec, stagename, hour, min);
+      else
+         sprintf(ln+i, "BEGIN STAGE %d-%d-%d at %02d:%02d\n",
+                 sec, subsec, subsubsec, hour, min);
+   }
+   else
+      sprintf(ln+i, "DONE  STAGE %d-%d-%d at %02d:%02d\n",
+              sec, subsec, subsubsec, hour, min);
+   if (START)
+   {
+      if (fp0)
+         fprintf(fp0, "\n\n");
+      if (fp1)
+         fprintf(fp1, "\n\n");
+   }
+   if (fp0)
+      fprintf(fp0, ln);
+   if (fp1)
+      fprintf(fp1, ln);
+}
+
+int LnIsCont(char *ln)
+/*
+ * RETURNS: 1 if last non-whitespace char in ln is '\', and 0 otherwise
+ */
+{
+   int i;
+   for (i=0; ln[i]; i++);
+   for (i--; Mciswspace(ln[i]) && i > 0; i--);
+   return(ln[i] == '\\');
+}
+
+void GetuMMRES(char pre, int ID, int *muladd, int *lat,
+               int *mu, int *nu, int *ku)
+/*
+ * Reads the user index file to obtain the settings for user-contributed
+ * kernel ID
+ */
+{
+   char ln[1024];
+   char fnam[256], auth[256];
+   FILE *fp;
+   int i, n, itmp, id;
+
+   sprintf(ln, "../tune/blas/gemm/%ccases.dsc", pre);
+   fp = fopen(ln, "r");
+   assert(fp);
+   fgets(ln, 1024, fp);  /* skip comment line */
+   fgets(ln, 1024, fp);  /* get number of user cases line */
+   assert(sscanf(ln, " %d", &n) == 1);
+/*
+ * Now search file for ID
+ */
+   for (i=0; i < n; i++)
+   {
+      assert(fgets(ln, 1024, fp));
+      assert(sscanf(ln, " %d %d %d %d %d %d %d %d %d %d %s \"%[^\"]", &id,
+                    &itmp, &itmp, &itmp, &itmp, muladd, lat, mu, nu, ku,
+                    fnam, auth) == 12);
+      if (LnIsCont(ln))
+      {
+         assert(fgets(ln, 1024, fp));
+         assert(fgets(ln, 1024, fp));
+      }
+      if (id == ID)
+         break;
+   }
+   assert(id == ID);
+   fclose(fp);
+}
 void GetMMRES(char pre, int *muladd, int *lat, int *nb, int *pref,
               int *mu, int *nu, int *ku, int *ff, int *iff, int *nf,
               double *mf, int *icase, char *ufile, char *auth, double *umf)
@@ -393,11 +485,102 @@ void GetMMRES(char pre, int *muladd, int *lat, int *nb, int *pref,
    fclose(fp);
 }
 
-void GoToTown(int ARCHDEF, int L1DEF)
+/*
+ *             Automatically Tuned Linear Algebra Software v3.9.51
+ *                    (C) Copyright 1999 R. Clint Whaley
+ *
+ * Code contributers : R. Clint Whaley, Antoine P. Petitet
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions
+ * are met:
+ *   1. Redistributions of source code must retain the above copyright
+ *      notice, this list of conditions and the following disclaimer.
+ *   2. Redistributions in binary form must reproduce the above copyright
+ *      notice, this list of conditions, and the following disclaimer in the
+ *      documentation and/or other materials provided with the distribution.
+ *   3. The name of the ATLAS group or the names of its contributers may
+ *      not be used to endorse or promote products derived from this
+ *      software without specific written permission.
+ *
+ * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
+ * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
+ * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
+ * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE ATLAS GROUP OR ITS CONTRIBUTORS
+ * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
+ * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
+ * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
+ * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
+ * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
+ * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
+ * POSSIBILITY OF SUCH DAMAGE.
+ *
+ */
+
+int ATL_lcm(const int M, const int N)
+/*
+ * Returns least common multiple (LCM) of two positive integers M & N by
+ * computing greatest common divisor (GCD) and using the property that
+ * M*N = GCD*LCM.
+ */
+{
+   register int tmp, max, min, gcd=0;
+
+   if (M != N)
+   {
+      if (M > N) { max = M; min = N; }
+      else { max = N; min = M; }
+      if (min > 0)  /* undefined for negative numbers */
+      {
+         do  /* while (min) */
+         {
+            if ( !(min & 1) ) /* min is even */
+            {
+               if ( !(max & 1) ) /* max is also even */
+               {
+                  do
+                  {
+                     min >>= 1;
+                     max >>= 1;
+                     gcd++;
+                     if (min & 1) goto MinIsOdd;
+                  }
+                  while ( !(max & 1) );
+               }
+               do min >>=1 ; while ( !(min & 1) );
+            }
+/*
+ *          Once min is odd, halve max until it too is odd.  Then, use
+ *          property that gcd(max, min) = gcd(max, (max-min)/2)
+ *          for odd max & min
+ */
+MinIsOdd:
+            if (min != 1)
+            {
+               do  /* while (max >= min */
+               {
+                  max -= (max & 1) ? min : 0;
+                  max >>= 1;
+               }
+               while (max >= min);
+            }
+            else return( (M*N) / (1<<gcd) );
+            tmp = max;
+            max = min;
+            min = tmp;
+         }
+         while(tmp);
+      }
+      return( (M*N) / (max<<gcd) );
+   }
+   else return(M);
+}
+
+void GoToTown(int ARCHDEF, int L1DEF, int TuneLA)
 {
    const char TR[2] = {'N','T'};
    char prec[4] = {'d', 's', 'z', 'c'}, pre, upre, *typ;
-   char ln[512], tnam[256], ln2[512], ln3[512], fnam[128];
+   char ln[1024], tnam[256], ln2[512], ln3[512], fnam[128];
    char *mulinst, *peakstr, *peakstr2;
    int nprec=4;
    int iL1, lat, muladd, maused, latuse, lbnreg;
@@ -407,9 +590,10 @@ void GoToTown(int ARCHDEF, int L1DEF)
    int ffetch, ifetch, nfetch;
    int DefInstall=0;
    long imf;
-   int maU, latU, muU, nuU, kuU, il1mul, pfA;
+   int idU, maU, latU, muU, nuU, kuU, il1mul, pfA;
    double mfU, mf4x1, mf4x4, mf, mfp, mmmf, mfpeak[2], l1mul;
    FILE *fp, *fpsum, *fpabr;
+   ATL_mmnode_t *mmp;
 
    PrintBanner("INSTALL_LOG/SUMMARY.LOG", 1, 0, 0, 0);
    fpsum = fopen("INSTALL_LOG/SUMMARY.LOG", "a");
@@ -425,8 +609,8 @@ void GoToTown(int ARCHDEF, int L1DEF)
    if (ARCHDEF)
       DefInstall = !system("make IArchDef.grd\n");
 
-   ATL_mprintf(2, stdout, fpsum,
-               "\n\nIN STAGE 1 INSTALL:  SYSTEM PROBE/AUX COMPILE\n");
+   PrintStartStop(stdout, fpsum, 0, 1, 1, 0, 0, "SYSTEM PROBE/AUX COMPILE");
+
 
    sprintf(ln2, "INSTALL_LOG/Stage1.log");
    PrintBanner("INSTALL_LOG/Stage1.log", 1, 1, 0, 0);
@@ -461,9 +645,9 @@ void GoToTown(int ARCHDEF, int L1DEF)
                   "         Register-register performance=%.2fMFLOPS\n",
                   mfpeak[ip]);
    }
+   PrintStartStop(stdout, fpsum, 0, 0, 1, 0, 0, NULL);
 
-   fprintf(stdout, "\n\nIN STAGE 2 INSTALL:  TYPE-DEPENDENT TUNING\n");
-   fprintf(fpsum , "\n\nIN STAGE 2 INSTALL:  TYPE-DEPENDENT TUNING\n");
+   PrintStartStop(stdout, fpsum, 0, 1, 2, 0, 0, "TYPE-DEPENDENT TUNING");
 
    for (ip=0; ip < nprec; ip++)
    {
@@ -495,20 +679,14 @@ void GoToTown(int ARCHDEF, int L1DEF)
          break;
       }
 
-      fprintf(stdout,
-              "\n\nSTAGE 2-%d: TUNING PREC=\'%c\' (precision %d of %d)\n",
-              ip+1, pre, ip+1, nprec);
-      fprintf(fpsum ,
-              "\n\nSTAGE 2-%d: TUNING PREC=\'%c\' (precision %d of %d)\n",
-              ip+1, pre, ip+1, nprec);
-      fprintf(stdout, "\n\n   STAGE 2-%d-1 : BUILDING BLOCK MATMUL TUNE\n",
-              ip+1);
-      fprintf(fpsum , "\n\n   STAGE 2-%d-1 : BUILDING BLOCK MATMUL TUNE\n",
-              ip+1);
+      sprintf(ln, "TUNING PREC=\'%c\' (precision %d of %d)", pre, ip+1, nprec);
+      PrintStartStop(stdout, fpsum, 0, 1, 2, ip+1, 0, ln);
+      PrintStartStop(stdout, fpsum, 3, 1, 2, ip+1, 1,
+                     "BUILDING BLOCK MATMUL TUNE");
 /*
  *    If necessary, install matmul for this type
  */
-      sprintf(fnam, "INSTALL_LOG/%cMMRES", pre);
+      sprintf(fnam, "INSTALL_LOG/%cMMRES.sum", pre);
       if (!FileExists(fnam))  /* need to run search or make link */
       {
          sprintf(ln2, "INSTALL_LOG/%cMMSEARCH.LOG", pre);
@@ -524,8 +702,9 @@ void GoToTown(int ARCHDEF, int L1DEF)
          ATL_Cassert(system(ln)==0, "BUILDING BLOCK MATMUL TUNE", ln2);
          PrintBanner(ln2, 0, 2, ip+1, 1);
       }
-      GetMMRES(pre, &muladd, &lat, &nb, &pfA, &mu, &nu, &ku, &ffetch, &ifetch,
-               &nfetch, &mf, &i, fnam, ln, &mfU);
+      mmp = ReadMMFileWithPath(pre, "INSTALL_LOG", "MMRES.sum");
+      assert(mmp);
+      nb = (mmp->next) ? mmp->next->nbB : mmp->nbB;
       #ifdef ATL_CPUMHZ
          fprintf(fpabr, "Clock_rate=%d Mhz\n", ATL_CPUMHZ);
          mfp = ATL_CPUMHZ;
@@ -540,32 +719,35 @@ void GoToTown(int ARCHDEF, int L1DEF)
       #endif
       fprintf(fpabr, "%% clock      MFLOP  ROUTINE/PROBLEM\n");
       fprintf(fpabr, "=======  =========  ===============\n");
-      if (mfU > mf)
+      if (mmp->next && mmp->next->mflop[0] > mmp->mflop[0])
       {
          ATL_mprintf(2, fpsum, stdout,
             "      The best matmul kernel was %s, NB=%d, written by %s\n",
-                    fnam, nb, ln);
+                    mmp->next->rout, nb,
+                    mmp->next->auth ? mmp->next->auth : "R. Clint Whaley");
          ATL_mprintf(2, fpsum, stdout,
             "      Performance: %.2fMFLOPS (%5.2f percent of %s)\n",
-                     mfU, (mfU/mfp)*100.0, peakstr);
+                     mmp->next->mflop[0], (mmp->next->mflop[0]/mfp)*100.0,
+                     peakstr);
          ATL_mprintf(2, fpsum, stdout,
-                     "        (Gen case got %.2fMFLOPS)\n", mf);
-         mmmf = mfU;
+                     "        (Gen case got %.2fMFLOPS)\n", mmp->mflop[0]);
+         mmmf = mmp->next->mflop[0];
       }
       else
       {
+         mmmf = mmp->mflop[0];
          ATL_mprintf(2, fpsum, stdout,
 "      %cL1MATMUL: lat=%d, nb=%d, pf=%d, mu=%d, nu=%d, ku=%d, if=%d, nf=%d;\n",
                  pre, lat, nb, pfA, mu, nu, ku, ifetch, nfetch);
          ATL_mprintf(2, fpsum, stdout,
             "                 Performance: %.2f (%5.2f percent of %s)\n",
-                     mf, (mf/mfp)*100.0, peakstr);
-         mmmf = mf;
+                     mmmf, (mmmf/mfp)*100.0, peakstr);
       }
       fprintf(fpabr, "%7.1f %10.1f  %s\n", (mmmf/mfp)*100.0, mmmf,
               "Chosen kgemm");
-      fprintf(fpabr, "%7.1f %10.1f  %s\n", (mf/mfp)*100.0, mf,
-              "Generated kgemm");
+      fprintf(fpabr, "%7.1f %10.1f  %s\n", (mmp->mflop[0]/mfp)*100.0,
+              mmp->mflop[0], "Generated kgemm");
+      KillAllMMNodes(mmp);
 
       sprintf(fnam, "INSTALL_LOG/%cNCNB", pre);
       if (!FileExists(fnam))
@@ -620,11 +802,11 @@ void GoToTown(int ARCHDEF, int L1DEF)
       ATL_Cassert(system(ln)==0, "BUILDING BLOCK MATMUL TUNE", ln2);
 
       fprintf(fpsum, "\n");
+      PrintStartStop(stdout, fpsum, 3, 0, 2, ip+1, 1, NULL);
 /*
  *    If necessary, find cacheedge
  */
-      ATL_mprintf(2, fpsum, stdout,
-                 "\n\n   STAGE 2-%d-2: CacheEdge DETECTION\n", ip+1);
+      PrintStartStop(stdout, fpsum, 3, 1, 2, ip+1, 2, "CacheEdge DETECTION");
       sprintf(ln2, "INSTALL_LOG/%cMMCACHEEDGE.LOG", pre);
       if (!FileExists("INSTALL_LOG/atlas_cacheedge.h"))
       {
@@ -674,12 +856,11 @@ void GoToTown(int ARCHDEF, int L1DEF)
          fprintf(fpsum, "      %c%cNKB set to %d bytes\n", pre, upre, i);
          fclose(fp);
       }
+      PrintStartStop(stdout, fpsum, 3, 0, 2, ip+1, 2, NULL);
 /*
  *    If necessary, determine Xover for this data type
  */
-      ATL_mprintf(2, fpsum, stdout,
-                 "\n\n   STAGE 2-%d-3: LARGE/SMALL CASE CROSSOVER DETECTION\n",
-                 ip+1);
+      PrintStartStop(stdout, fpsum, 3, 1, 2, ip+1, 3, "SMALL/LARGE CROSSOVER");
       sprintf(fnam, "INSTALL_LOG/%cXover.h", pre);
       if (!FileExists(fnam))  /* need to run Xover tests */
       {
@@ -697,11 +878,11 @@ void GoToTown(int ARCHDEF, int L1DEF)
          fprintf(stdout, "      done.\n");
          fprintf(fpsum , "      done.\n");
       }
+      PrintStartStop(stdout, fpsum, 3, 0, 2, ip+1, 3, NULL);
 
       sprintf(ln2, "INSTALL_LOG/%cL3TUNE.LOG", pre);
       PrintBanner(ln2, 1, 2, ip+1, 5);
-      ATL_mprintf(2, stdout, fpsum,
-                 "\n\n   STAGE 2-%d-4: LEVEL 3 BLAS TUNE\n", ip+1);
+      PrintStartStop(stdout, fpsum, 3, 1, 2, ip+1, 4, "L3BLAS TUNE");
       if (pre == 's' || pre == 'd')
       {
          sprintf(ln, "%s INSTALL_LOG/atlas_%ctrsmXover.h pre=%c %s %s\n",
@@ -719,53 +900,79 @@ void GoToTown(int ARCHDEF, int L1DEF)
       fprintf(stdout, ln);
       ATL_Cassert(system(ln)==0, "L3BLAS TUNING", ln2);
       PrintBanner(ln2, 0, 2, ip+1, 5);
-      ATL_mprintf(2, fpsum, stdout, "      done.\n");
+      PrintStartStop(stdout, fpsum, 3, 0, 2, ip+1, 4, "L3BLAS TUNE");
 
 
-      ATL_mprintf(2, fpsum, stdout,"\n\n   STAGE 2-%d-5: GEMV TUNE\n", ip+1);
-      sprintf(fnam, "INSTALL_LOG/%cMVRES", pre);
+      PrintStartStop(stdout, fpsum, 3, 1, 2, ip+1, 5, "GEMV TUNE");
+      sprintf(fnam, "INSTALL_LOG/%cMVNK.sum", pre);
       if (!FileExists(fnam))
       {
-         sprintf(ln2, "INSTALL_LOG/%cMVTUNE.LOG", pre);
+         sprintf(ln2, "INSTALL_LOG/%cMVNTUNE.LOG", pre);
          PrintBanner(ln2, 1, 2, ip+1, 7);
          sprintf(ln, "%s %s pre=%c %s %s\n", fmake, fnam, pre, redir, ln2);
          fprintf(stdout, ln);
-         ATL_Cassert(system(ln)==0, "MVTUNE", ln2);
-         ATL_Cassert(FileIsThere(fnam), "MVTUNE", ln2);
+         ATL_Cassert(system(ln)==0, "MVNTUNE", ln2);
+         ATL_Cassert(FileIsThere(fnam), "MVNTUNE", ln2);
          PrintBanner(ln2, 0, 2, ip+1, 7);
       }
-      fp = fopen(fnam, "r");
-      ATL_Cassert(fp, "MVTUNE", NULL);
-      ATL_Cassert(fgets(ln, 256, fp), "MVTUNE", NULL);
-      ATL_Cassert(sscanf(ln, " %d %d %d %d %d %lf %s \"%[^\"]", &j, &i,
-                  &mu, &nu, &il1mul, &mf, ln3, tnam) == 8, "MVTUNE", NULL);
-      ATL_mprintf(2, fpsum, stdout,
-                  "      gemvN : chose routine %d:%s written by %s\n",
-                  j, ln3, tnam);
-      ATL_mprintf(2, fpsum, stdout,
-         "              Yunroll=%d, Xunroll=%d, using %d percent of L1\n",
-                  mu, nu, il1mul);
-      ATL_mprintf(2, fpsum, stdout,
-"              Performance = %.2f (%5.2f of copy matmul, %5.2f of %s)\n",
-              mf, (mf/mmmf)*100.0, (mf / mfp)*100.0, peakstr2);
-      fprintf(fpabr, "%7.1f %10.1f  %s\n", (mf/mfp)*100.0, mf, "kgemvN");
-      ATL_Cassert(fgets(ln, 256, fp), "MVTUNE", NULL);
-      ATL_Cassert(sscanf(ln, " %d %d %d %d %d %lf %s \"%[^\"]", &j, &i,
-                  &mu, &nu, &il1mul, &mf, ln3, tnam) == 8, "MVTUNE", NULL);
-      fclose(fp);
-      ATL_mprintf(2, fpsum, stdout,
-                  "      gemvT : chose routine %d:%s written by %s\n",
-                  j, ln3, tnam);
-      ATL_mprintf(2, fpsum, stdout,
-         "              Yunroll=%d, Xunroll=%d, using %d percent of L1\n",
-                  mu, nu, il1mul);
-      ATL_mprintf(2, fpsum, stdout,
-"              Performance = %.2f (%5.2f of copy matmul, %5.2f of %s)\n",
-              mf, (mf/mmmf)*100.0, (mf / mfp)*100.0, peakstr2);
-      fprintf(fpabr, "%7.1f %10.1f  %s\n", (mf/mfp)*100.0, mf, "kgemvT");
+      {
+         ATL_mvnode_t *kb, *kp;
 
-      ATL_mprintf(2, fpsum, stdout,"\n\n   STAGE 2-%d-6: GER TUNE\n", ip+1);
-      sprintf(fnam, "INSTALL_LOG/%cR1RES", pre);
+         kb = ReadMVFile(fnam);
+         ATL_Cassert(kb, "MVNTUNE", NULL);
+         ATL_MVSplitContexts(kb, &kb, NULL, NULL, NULL);
+         kp = ATL_LastMVNode(kb);
+         ATL_mprintf(2, fpsum, stdout,
+              "      gemvN : main kernel %d:%s written by %s\n", kp->ID,
+                     kp->rout?kp->rout : "Generated",
+                     kp->auth?kp->auth : "R. Clint Whaley");
+         ATL_mprintf(2, fpsum, stdout,
+                     "            mu=%d, nu=%d, using %d Cache Elements\n",
+                     kp->MU, kp->NU, kp->CacheElts);
+         mf = Mmax(kp->mflop[0], kp->mflop[1]);
+         mf = Mmax(mf, kp->mflop[2]);
+         KillAllMVNodes(kb);
+         ATL_mprintf(2, fpsum, stdout,
+"              Performance = %.2f (%5.2f of copy matmul, %5.2f of %s)\n",
+                 mf, (mf/mmmf)*100.0, (mf / mfp)*100.0, peakstr2);
+         fprintf(fpabr, "%7.1f %10.1f  %s\n", (mf/mfp)*100.0, mf, "kgemvN");
+      }
+      sprintf(fnam, "INSTALL_LOG/%cMVTK.sum", pre);
+      if (!FileExists(fnam))
+      {
+         sprintf(ln2, "INSTALL_LOG/%cMVTTUNE.LOG", pre);
+         PrintBanner(ln2, 1, 2, ip+1, 7);
+         sprintf(ln, "%s %s pre=%c %s %s\n", fmake, fnam, pre, redir, ln2);
+         fprintf(stdout, ln);
+         ATL_Cassert(system(ln)==0, "MVTTUNE", ln2);
+         ATL_Cassert(FileIsThere(fnam), "MVTTUNE", ln2);
+         PrintBanner(ln2, 0, 2, ip+1, 7);
+      }
+      {
+         ATL_mvnode_t *kb, *kp;
+
+         kb = ReadMVFile(fnam);
+         ATL_Cassert(kb, "MVTTUNE", NULL);
+         ATL_MVSplitContexts(kb, &kb, NULL, NULL, NULL);
+         kp = ATL_LastMVNode(kb);
+         ATL_mprintf(2, fpsum, stdout,
+              "      gemvT : main kernel %d:%s written by %s\n", kp->ID,
+                     kp->rout?kp->rout : "Generated",
+                     kp->auth?kp->auth : "R. Clint Whaley");
+         ATL_mprintf(2, fpsum, stdout,
+                     "            mu=%d, nu=%d, using %d Cache Elements\n",
+                     kp->MU, kp->NU, kp->CacheElts);
+         mf = Mmax(kp->mflop[0], kp->mflop[1]);
+         mf = Mmax(mf, kp->mflop[2]);
+         KillAllMVNodes(kb);
+         ATL_mprintf(2, fpsum, stdout,
+"              Performance = %.2f (%5.2f of copy matmul, %5.2f of %s)\n",
+                 mf, (mf/mmmf)*100.0, (mf / mfp)*100.0, peakstr2);
+         fprintf(fpabr, "%7.1f %10.1f  %s\n", (mf/mfp)*100.0, mf, "kgemvT");
+      }
+      PrintStartStop(stdout, fpsum, 3, 0, 2, ip+1, 5, "GEMV TUNE");
+      PrintStartStop(stdout, fpsum, 3, 1, 2, ip+1, 6, "GER TUNE");
+      sprintf(fnam, "INSTALL_LOG/%cR2K.sum", pre);
       if (!FileExists(fnam))
       {
          sprintf(ln2, "INSTALL_LOG/%cR1TUNE.LOG", pre);
@@ -775,25 +982,36 @@ void GoToTown(int ARCHDEF, int L1DEF)
          ATL_Cassert(system(ln)==0, "R1TUNE", ln2);
          ATL_Cassert(FileIsThere(fnam), "R1TUNE", ln2);
          PrintBanner(ln2, 0, 2, ip+1, 7);
+         fnam[14] = '1';
       }
-      fp = fopen(fnam, "r");
-      ATL_Cassert(fp, "R1TUNE", NULL);
-      ATL_Cassert(fgets(ln, 256, fp), "R1TUNE", NULL);
-      ATL_Cassert(sscanf(ln, " %d %d %d %d %d %lf %s \"%[^\"]", &j, &i, &mu,
-                         &nu, &il1mul, &mf, ln3, tnam) == 8, "R1TUNE", NULL);
-      fclose(fp);
-      ATL_mprintf(2, fpsum, stdout,
-              "      ger : chose routine %d:%s written by %s\n", j, ln3, tnam);
-      ATL_mprintf(2, fpsum, stdout,
-              "            mu=%d, nu=%d, using %5.2f percent of L1 Cache\n",
-              mu, nu, il1mul*0.01);
-      ATL_mprintf(2, fpsum, stdout,
+      {
+         ATL_r1node_t *r1b, *r1B;
+         r1b = ReadR1File(fnam);
+         ATL_Cassert(r1b, "R1TUNE", NULL);
+         ATL_R1SplitContexts(r1b, &r1B, NULL, NULL, NULL);
+         r1b = r1B;
+         r1B = ATL_LastR1Node(r1B);
+         ATL_mprintf(2, fpsum, stdout,
+              "      ger : main kernel %d:%s written by %s\n", r1B->ID,
+                     r1B->rout?r1B->rout : "Generated",
+                     r1B->auth?r1B->auth : "R. Clint Whaley");
+         ATL_mprintf(2, fpsum, stdout,
+                     "            mu=%d, nu=%d, using %d Cache Elements\n",
+                     r1B->MU, r1B->NU, r1B->CacheElts);
+         mf = Mmax(r1B->mflop[0], r1B->mflop[1]);
+         mf = Mmax(mf, r1B->mflop[2]);
+         KillAllR1Nodes(r1b);
+         ATL_mprintf(2, fpsum, stdout,
 "              Performance = %.2f (%5.2f of copy matmul, %5.2f of %s)\n",
-              mf, (mf/mmmf)*100.0, (mf / mfp)*100.0, peakstr2);
-      fprintf(fpabr, "%7.1f %10.1f  %s\n", (mf/mfp)*100.0, mf, "kger");
-      fclose(fpabr);
+                 mf, (mf/mmmf)*100.0, (mf / mfp)*100.0, peakstr2);
+         fprintf(fpabr, "%7.1f %10.1f  %s\n", (mf/mfp)*100.0, mf, "kger");
+         fclose(fpabr);
+      }
+      PrintStartStop(stdout, fpsum, 3, 0, 2, ip+1, 6, "GER TUNE");
+      PrintStartStop(stdout, fpsum, 0, 0, 2, 0, 0, "TYPE-DEPENDENT TUNING");
    }
-   ATL_mprintf(2, fpsum, stdout,"\n\nSTAGE 3: GENERAL LIBRARY BUILD\n");
+   PrintStartStop(stdout, fpsum, 0, 0, 2, 0, 0, "TYPE-DEPENDENT TUNING");
+   PrintStartStop(stdout, fpsum, 0, 1, 3, 0, 0, "GENERAL LIBRARY BUILD");
 
    sprintf(ln2, "INSTALL_LOG/LIBBUILD.LOG");
    PrintBanner(ln2, 1, 3, 1, 1);
@@ -802,28 +1020,75 @@ void GoToTown(int ARCHDEF, int L1DEF)
    ATL_Cassert(system(ln)==0, "LIBRARY BUILD", ln2);
    ATL_Cassert(FileIsThere(fnam), "LIBRARY BUILD", ln2);
    PrintBanner(ln2, 0, 3, 1, 1);
+   PrintStartStop(stdout, fpsum, 0, 0, 3, 0, 0, "GENERAL LIBRARY BUILD");
 
-   ATL_mprintf(2, fpsum, stdout,"\n\nSTAGE 4: POST-BUILD TUNING\n");
+   PrintStartStop(stdout, fpsum, 0, 1, 4, 0, 0, "POST-BUILD TUNING");
    sprintf(ln2, "INSTALL_LOG/POSTTUNE.LOG");
    PrintBanner(ln2, 1, 4, 1, 1);
+   PrintStartStop(stdout, fpsum, 3, 1, 4, 1, 1, "TRSM TUNE");
    sprintf(ln, "%s IPostTune %s %s\n", fmake, redir, ln2);
    fprintf(stdout, ln);
    ATL_Cassert(system(ln)==0, "POST-BUILD TUNE", ln2);
+   PrintStartStop(stdout, fpsum, 3, 0, 4, 1, 0, NULL);
    ATL_Cassert(FileIsThere(fnam), "POST-BUILD TUNE", ln2);
    PrintBanner(ln2, 0, 4, 1, 1);
 
-   fprintf(stdout, "   done.\n");
-   fprintf(fpsum,  "   done.\n");
-
 #ifdef ATL_NCPU
-   ATL_mprintf(2, stdout, fpsum, "\n\nSTAGE 4: Threading install\n");
+   PrintStartStop(stdout, fpsum, 3, 1, 4, 2, 0, "THREADING TUNE");
+   sprintf(ln2, "INSTALL_LOG/PTTUNE.LOG");
+   PrintBanner(ln2, 1, 4, 2, 0);
+   sprintf(ln, "%s IPTtune %s %s\n", fmake, redir, ln2);
+   fprintf(stdout, ln);
+   ATL_Cassert(system(ln)==0, "THREADING TUNE", ln2);
+   PrintStartStop(stdout, fpsum, 3, 0, 4, 2, 0, "THREADING TUNE");
 
+   PrintStartStop(stdout, fpsum, 3, 1, 4, 2, 1, "THREADING BUILD");
    sprintf(ln2, "INSTALL_LOG/LIBPTBUILD.LOG");
    PrintBanner(ln2, 1, 4, 2, 1);
    sprintf(ln, "%s IBuildPtlibs %s %s\n", fmake, redir, ln2);
    fprintf(stdout, ln);
    ATL_Cassert(system(ln)==0, "PTLIBRARY BUILD", ln2);
    PrintBanner(ln2, 0, 4, 2, 1);
+   PrintStartStop(stdout, fpsum, 3, 0, 4, 2, 1, "THREADING BUILD");
+#endif
+   if (TuneLA)
+   {
+      PrintStartStop(stdout, fpsum, 3, 1, 4, 3, 0, "LAPACK TUNING");
+      for (ip=0; ip < nprec; ip++)
+      {
+         pre = prec[ip];
+         sprintf(ln2, "%cLAPACK TUNING", pre);
+         PrintStartStop(stdout, fpsum, 6, 1, 4, 3, ip+1, ln2);
+         sprintf(ln2, "INSTALL_LOG/%cLATUNE.LOG", pre);
+         PrintBanner(ln2, 1, 4, 3, ip+1);
+         fprintf(stdout, ln);
+         sprintf(ln, "%s ILATune pre=%c %s %s\n", fmake, pre, redir, ln2);
+         ATL_Cassert(system(ln)==0, "LAPACK TUNE", ln2);
+         PrintStartStop(stdout, fpsum, 6, 0, 4, 3, ip+1, NULL);
+         PrintBanner(ln2, 0, 4, 3, ip+1);
+      }
+      PrintStartStop(stdout, fpsum, 3, 0, 4, 3, 0, "LAPACK TUNING");
+   }
+   PrintStartStop(stdout, fpsum, 0, 0, 4, 0, 0, "POST-BUILD TUNING");
+   PrintStartStop(stdout, fpsum, 0, 1, 5, 0, 0, "FINAL LIBRARY UPDATE");
+   PrintStartStop(stdout, fpsum, 3, 1, 5, 1, 0, "FINAL STATIC LIBRARY UPDATE");
+   sprintf(ln2, "INSTALL_LOG/LIBUPDATE.LOG");
+   PrintBanner(ln2, 1, 5, 1, 1);
+   sprintf(ln, "%s IBuildLibs IBuildPtlibs0 %s %s\n", fmake, redir, ln2);
+   fprintf(stdout, ln);
+   ATL_Cassert(system(ln)==0, "STATIC LIBRARY UPDATE", ln2);
+   PrintBanner(ln2, 0, 5, 1, 1);
+   PrintStartStop(stdout, fpsum, 3, 0, 5, 1, 0, "FINAL STATIC LIBRARY UPDATE");
+#ifdef ATL_DYLIBS
+   PrintStartStop(stdout, fpsum, 3, 1, 5, 2, 0,
+                  "DYNAMIC/SHARED LIBRARY UPDATE");
+   sprintf(ln2, "INSTALL_LOG/LIBDYBUILD.LOG");
+   PrintBanner(ln2, 1, 5, 2, 1);
+   sprintf(ln, "%s IBuildDyLibs %s %s\n", fmake, redir, ln2);
+   fprintf(stdout, ln);
+   ATL_Cassert(system(ln)==0, "DYLIBRARY BUILD", ln2);
+   PrintBanner(ln2, 0, 5, 2, 1);
+   PrintStartStop(stdout, fpsum, 3, 0, 5, 2, 0, NULL);
 #endif
 
    fprintf(stdout, "\n\n\n\n");
@@ -840,11 +1105,11 @@ void PrintUsage(char *nam)
    exit(-1);
 }
 
-void GetFlags(int nargs, char *args[], int *ARCHDEF, int *L1DEF)
+void GetFlags(int nargs, char *args[], int *ARCHDEF, int *L1DEF, int *TuneLA)
 {
    int i;
 
-   *L1DEF = 0;
+   *TuneLA = *L1DEF = 0;
    *ARCHDEF = 1;
    for (i=1; i < nargs; i++)
    {
@@ -856,6 +1121,11 @@ void GetFlags(int nargs, char *args[], int *ARCHDEF, int *L1DEF)
             PrintUsage(args[0]);
          *ARCHDEF = atoi(args[i]);
          break;
+      case 'l':
+         if (++i > nargs)
+            PrintUsage(args[0]);
+         *TuneLA = atoi(args[i]);
+         break;
       case '1':
          if (++i > nargs)
             PrintUsage(args[0]);
@@ -867,10 +1137,10 @@ void GetFlags(int nargs, char *args[], int *ARCHDEF, int *L1DEF)
    }
 }
 
-main(int nargs, char *args[])
+int main(int nargs, char *args[])
 {
-   int L1DEF, ARCHDEF;
-   GetFlags(nargs, args, &ARCHDEF, &L1DEF);
-   GoToTown(ARCHDEF, L1DEF);
-   exit(0);
+   int L1DEF, ARCHDEF, TuneLA;
+   GetFlags(nargs, args, &ARCHDEF, &L1DEF, &TuneLA);
+   GoToTown(ARCHDEF, L1DEF, TuneLA);
+   return(0);
 }
