@@ -1,5 +1,5 @@
 /*
- *             Automatically Tuned Linear Algebra Software v3.8.4
+ *             Automatically Tuned Linear Algebra Software v3.9.51
  *                    (C) Copyright 1997 R. Clint Whaley
  *
  * Redistribution and use in source and binary forms, with or without
@@ -27,6 +27,11 @@
  * POSSIBILITY OF SUCH DAMAGE.
  *
  */
+#if defined(ATL_AFFINITY) || defined(ATL_NOAFFINITY)
+   #ifndef ATL_USEPTHREADS
+      #define ATL_USEPTHREADS
+   #endif
+#endif
 
 /* #define TEST_GPMM */
 #include <stdio.h>
@@ -37,9 +42,43 @@
 #include "atlas_lvl3.h"
 #include "atlas_tst.h"
 #include "cblas.h"
+#ifdef ATL_USEPTHREADS
+   #include "atlas_tlvl3.h"
+#endif
+#if defined(ATL_RKXOVER) || defined(TEST_RANKK)
+   #define TRUST_BIGNORK
+   void Mjoin(PATL,bignork_mm)
+      (const enum ATLAS_TRANS TA, const enum ATLAS_TRANS TB,
+       const int M, const int N, const int K, const SCALAR alpha,
+       const TYPE *A, const int lda, const TYPE *B, const int ldb,
+       const SCALAR beta, TYPE *C, const int ldc);
+   #ifdef TCPLX
+      #error "ATLAS's rank-K update does not yet support complex types!"
+   #endif
+#endif
+#if defined(ATL_RKXOVER) && !defined(TEST_RANKK)
+   #define TEST_RANKK
+#endif
 #ifdef TEST_GPMM
    #include "atlas_pkblas.h"
 #endif
+#ifdef TEST_RANKK
+   #include Mstr(Mjoin(Mjoin(atlas_,PRE),vkbmms.h))
+int Mjoin(PATL,vrankK)(const enum ATLAS_TRANS TA, const enum ATLAS_TRANS TB,
+                       ATL_CINT M, ATL_CINT N, ATL_CINT K, const SCALAR alpha,
+                       const TYPE *A, ATL_CINT lda, const TYPE *B,
+                       ATL_CINT ldb, const SCALAR beta, TYPE *C, ATL_CINT ldc);
+   #ifdef ATL_RKXOVER
+      FILE *fprk=NULL;
+      int speedup_rk[ATL_vMAX_KB+1], speedup_rk_b0[ATL_vMAX_KB+1];
+      int MAX_vKB = ATL_vMAX_KB;
+   #endif
+#elif defined(TEST_GEMM_RK)
+int Mjoin(PATL,gemm_rK)(const enum ATLAS_TRANS TA, const enum ATLAS_TRANS TB,
+                        ATL_CINT M, ATL_CINT N, ATL_CINT K, const SCALAR alpha,
+                        const TYPE *A, ATL_CINT lda, const TYPE *B,
+                        ATL_CINT ldb, const SCALAR beta, TYPE *C, ATL_CINT ldc);
+#endif
 
 #ifndef L2SIZE
    #define L2SIZE 4194304
@@ -81,17 +120,53 @@ void matgen(int M, int N, TYPE *A, int lda, int seed)
    for (j=N; j; j--)
    {
       for (i=0; i != M; i++) A[i] = dumb_rand();
+      for (; i < lda; i++) A[i] = -9e27;
       A += lda;
    }
 }
 
+#ifdef ATL_NANC
+void matnan(int M, int N, TYPE *C, int ldc, int seed)
+/*
+ * Fills matrix with NaNs
+ */
+{
+   int i, j;
+   unsigned int *ip;
+   TYPE MyNaN;
+   #ifdef TCPLX
+      const int m=M+M, ldC=ldc+ldc;
+   #else
+      const int m=M, ldC=ldc;
+   #endif
+#if 0
+   ip = (unsigned int*) &MyNaN;
+   ip[0] = 1;
+   ip[1] = 2047 << (31-11);
+#else
+   MyNaN = 0.0/0.0;
+#endif
+   for (j=0; j < N; j++, C += ldC)
+   {
+      for (i=0; i < m; i++)
+         C[i] = MyNaN;
+   }
+}
+#endif
+
 #if defined(TRUST_C)
    #define trusted_gemm(TA, TB, m, n, k, al, A, lda, B, ldb, be, C, ldc) \
       Mjoin(Mjoin(cblas_,PRE),gemm) \
          (CblasColMajor, TA, TB, m, n, k, al, A, lda, B, ldb, be, C, ldc)
+#elif defined(ATL_NOAFFINITY) || defined(ATL_AFFINITY)
+   #define trusted_gemm(TA, TB, m, n, k, al, A, lda, B, ldb, be, C, ldc) \
+      Mjoin(PATL,tgemm)(TA, TB, m, n, k, al, A, lda, B, ldb, be, C, ldc)
 #elif defined(TRUST_SMALL)
    #define trusted_gemm(TA, TB, m, n, k, al, A, lda, B, ldb, be, C, ldc) \
       Mjoin(PATL,small_mm)(TA, TB, m, n, k, al, A, lda, B, ldb, be, C, ldc)
+#elif defined(TRUST_BIGNORK)
+   #define trusted_gemm(TA, TB, m, n, k, al, A, lda, B, ldb, be, C, ldc) \
+      Mjoin(PATL,bignork_mm)(TA, TB, m, n, k, al, A, lda, B, ldb, be, C, ldc)
 #elif defined(TRUST_BIG)
    #define trusted_gemm(TA, TB, m, n, k, al, A, lda, B, ldb, be, C, ldc) \
       Mjoin(PATL,big_mm)(TA, TB, m, n, k, al, A, lda, B, ldb, be, C, ldc)
@@ -101,6 +176,9 @@ void matgen(int M, int N, TYPE *A, int lda, int seed)
 #elif defined(TRUST_JIT)
    #define trusted_gemm(TA, TB, m, n, k, al, A, lda, B, ldb, be, C, ldc) \
       Mjoin(PATL,mmJITcp)(TA, TB, m, n, k, al, A, lda, B, ldb, be, C, ldc)
+#elif defined(ATL_USEPTHREADS) && !defined(TEST_F77)
+   #define trusted_gemm(TA, TB, m, n, k, al, A, lda, B, ldb, be, C, ldc) \
+      Mjoin(PATL,gemm)(TA, TB, m, n, k, al, A, lda, B, ldb, be, C, ldc)
 #else
    #define trusted_gemm(TA, TB, m, n, k, al, A, lda, B, ldb, be, C, ldc) \
       Mjoin(PATL,f77gemm)(TA, TB, m, n, k, al, A, lda, B, ldb, be, C, ldc)
@@ -110,6 +188,12 @@ void matgen(int M, int N, TYPE *A, int lda, int seed)
    #define test_gemm(TA, TB, m, n, k, al, A, lda, B, ldb, be, C, ldc) \
       Mjoin(PATL,gpmm)(PackGen, TA, PackGen, TB, PackGen, m, n, k, al, \
                        A, 0, 0, lda, B, 0, 0, ldb, be, C, 0, 0, ldc)
+#elif defined(ATL_NOAFFINITY) || defined(ATL_AFFINITY)
+   #define test_gemm(TA, TB, m, n, k, al, A, lda, B, ldb, be, C, ldc) \
+      Mjoin(PATL,tgemm)(TA, TB, m, n, k, al, A, lda, B, ldb, be, C, ldc)
+#elif defined(TEST_F77)
+   #define test_gemm(TA, TB, m, n, k, al, A, lda, B, ldb, be, C, ldc) \
+      Mjoin(PATL,f77gemm)(TA, TB, m, n, k, al, A, lda, B, ldb, be, C, ldc)
 #elif defined(TEST_BIG)
    #define test_gemm(TA, TB, m, n, k, al, A, lda, B, ldb, be, C, ldc) \
       Mjoin(PATL,big_mm)(TA, TB, m, n, k, al, A, lda, B, ldb, be, C, ldc)
@@ -125,9 +209,29 @@ void matgen(int M, int N, TYPE *A, int lda, int seed)
 #elif defined(TEST_JIT)
    #define test_gemm(TA, TB, m, n, k, al, A, lda, B, ldb, be, C, ldc) \
       Mjoin(PATL,mmJITcp)(TA, TB, m, n, k, al, A, lda, B, ldb, be, C, ldc)
+#elif defined(TEST_RANKK)
+   #define test_gemm(TA, TB, m, n, k, al, A, lda, B, ldb, be, C, ldc) \
+   { \
+      if (k <= ATL_vMAX_KB) \
+      { \
+         ATL_assert(!Mjoin(PATL,vrankK)(TA, TB, m, n, k, al, A, lda, B, ldb, \
+                                        be, C, ldc));\
+      } \
+      else \
+      { \
+         fprintf(stderr, "maxK exceeded at %d\n", k); \
+         exit(k); \
+      } \
+   }
+#elif defined(TEST_GEMM_RK)
+   #define test_gemm(TA, TB, m, n, k, al, A, lda, B, ldb, be, C, ldc) \
+      ATL_assert(!Mjoin(PATL,gemm_rK)(TA, TB, m, n, k, al, A, lda, B, ldb, be, C, ldc))
 #elif defined(TEST_ALIASED)
    #define test_gemm(TA, TB, m, n, k, al, A, lda, B, ldb, be, C, ldc) \
       Mjoin(PATL,aliased_gemm)(TA, TB, m, n, k, al, A, lda, B, ldb, be, C, ldc)
+#elif defined(ATL_USEPTHREADS)
+   #define test_gemm(TA, TB, m, n, k, al, A, lda, B, ldb, be, C, ldc) \
+      Mjoin(PATL,tgemm)(TA, TB, m, n, k, al, A, lda, B, ldb, be, C, ldc)
 #else
    #define test_gemm(TA, TB, m, n, k, al, A, lda, B, ldb, be, C, ldc) \
       Mjoin(PATL,gemm)(TA, TB, m, n, k, al, A, lda, B, ldb, be, C, ldc)
@@ -147,7 +251,7 @@ int mmcase(int TEST, int CACHESIZE, char TA, char TB, int M, int N, int K,
    #define MBETA *beta, beta[1]
    char *form="%4d   %c   %c %4d %4d %4d  %5.1f %5.1f  %5.1f %5.1f  %6.2f %6.1f %4.2f   %3s\n";
 #endif
-   int ii, jj, i, j=0, PASSED, nerrs;
+   int ii, jj, i, j=0, PASSED, nerrs, Na, Nb;
    double t0, t1, t2, t3, mflop;
    TYPE maxval, f1, ferr;
    static TYPE feps=0.0;
@@ -158,30 +262,40 @@ int mmcase(int TEST, int CACHESIZE, char TA, char TB, int M, int N, int K,
 
    if (!TEST) D = C;
    /*if (nL2) L2 = malloc(nL2*sizeof(int));*/
-   l2ret = ATL_flushcache( CACHESIZE );
+   #if !ATL_LINEFLUSH
+      l2ret = ATL_flushcache( CACHESIZE );
+   #endif
    if (TA == 'n' || TA == 'N')
    {
       matgen(M, K, A, lda, K*1112);
       TAc = AtlasNoTrans;
+      Na = K;
    }
    else
    {
       matgen(K, M, A, lda, K*1112);
       if (TA == 'c' || TA == 'C') TAc = AtlasConjTrans;
       else TAc = AtlasTrans;
+      Na = M;
    }
    if (TB == 'n' || TB == 'N')
    {
       matgen(K, N, B, ldb, N*2238);
       TBc = AtlasNoTrans;
+      Nb = N;
    }
    else
    {
       matgen(N, K, B, ldb, N*2238);
       if (TB == 'c' || TB == 'C') TBc = AtlasConjTrans;
       else TBc = AtlasTrans;
+      Nb = K;
    }
+#ifdef ATL_NANC
+   matnan(M, N, C, ldc, M*N);
+#else
    matgen(M, N, C, ldc, M*N);
+#endif
 
 #ifdef DEBUG
    printmat("A0", M, K, A, lda);
@@ -197,7 +311,13 @@ int mmcase(int TEST, int CACHESIZE, char TA, char TB, int M, int N, int K,
      }*/
 
    /* invalidate L2 cache */
-   l2ret = ATL_flushcache( -1 );
+   #if !ATL_LINEFLUSH
+      l2ret = ATL_flushcache( -1 );
+   #else
+      ATL_flushCacheByAddr(ldc*N, C);
+      ATL_flushCacheByAddr(ldb*Nb, B);
+      ATL_flushCacheByAddr(lda*Na, A);
+   #endif
 
    t0 = time00();
    trusted_gemm(TAc, TBc, M, N, K, alpha, A, lda, B, ldb, beta, C, ldc);
@@ -216,11 +336,20 @@ int mmcase(int TEST, int CACHESIZE, char TA, char TB, int M, int N, int K,
 #endif
 
 #ifndef TIMEONLY
-   matgen(M, N, D, ldd, M*N);
+   #ifdef ATL_NANC
+      matnan(M, N, D, ldd, M*N);
+   #else
+      matgen(M, N, D, ldd, M*N);
+   #endif
 
    /* invalidate L2 cache */
-   l2ret = ATL_flushcache( -1 );
-
+   #if !ATL_LINEFLUSH
+      l2ret = ATL_flushcache( -1 );
+   #else
+      ATL_flushCacheByAddr(ldc*N, D);
+      ATL_flushCacheByAddr(ldb*Nb, B);
+      ATL_flushCacheByAddr(lda*Na, A);
+   #endif
    t0 = time00();
    test_gemm(TAc, TBc, M, N, K, alpha, A, lda, B, ldb, beta, D, ldd);
 
@@ -271,6 +400,20 @@ int mmcase(int TEST, int CACHESIZE, char TA, char TB, int M, int N, int K,
       {
          for (i=0; i != M SHIFT; i++)
          {
+            #ifdef ATL_NANC
+               if (D[i] != D[i])
+               {
+                  nerrs++;
+                  PASSED = 0;
+                  printf("NaN in test_gemm at C(%d,%d)\n", i, j);
+               }
+               if (C[i] != C[i])
+               {
+                  nerrs++;
+                  PASSED = 0;
+                  printf("NaN in trusted_gemm at C(%d,%d)\n", i, j);
+               }
+            #endif
             f1 = D[i] - C[i];
             if (f1 < 0.0) f1 = -f1;
             if (f1 > ferr)
@@ -285,6 +428,13 @@ int mmcase(int TEST, int CACHESIZE, char TA, char TB, int M, int N, int K,
                   jj = j+1;
                }
             }
+            else if (D[i] != D[i])
+            {
+               nerrs++;
+               PASSED = 0;
+               pc = "NaN";
+               maxval = D[i];
+            }
          }
          D += ldd SHIFT;
          C += ldc SHIFT;
@@ -296,17 +446,28 @@ int mmcase(int TEST, int CACHESIZE, char TA, char TB, int M, int N, int K,
    if (t1 == t2) t3 = 1.0;
    else if (t2 != 0.0) t3 = t1/t2;
    else t3 = 0.0;
+   #ifdef ATL_RKXOVER
+      if (K <= ATL_vMAX_KB)
+      {
+         if (SCALAR_IS_ZERO(beta))
+            speedup_rk_b0[K] = (t3 >= 1.0) ? t3*100 : 0;
+         else
+            speedup_rk[K] = (t3 >= 1.0) ? t3*100 : 0;
+      }
+   #endif
    printf(form, itst++, TA, TB, M, N, K, MALPH, MBETA, t2, mflop, t3, pc);
 #else
    itst++;
    PASSED = 1;
 #endif
    /*free(L2);*/
-   l2ret = ATL_flushcache( 0 );
+   #if !ATL_LINEFLUSH
+      l2ret = ATL_flushcache( 0 );
+   #endif
    return(PASSED);
 }
 
-int mmcase0(int MFLOP, int CACHESIZE, char TA, char TB, int M, int N, int K,
+int mmcase0(int MFLOP, size_t CACHESIZE, char TA, char TB, int M, int N, int K,
 	    SCALAR alpha, int lda, int ldb, SCALAR beta, int ldc)
 {
    char *pc;
@@ -321,16 +482,23 @@ int mmcase0(int MFLOP, int CACHESIZE, char TA, char TB, int M, int N, int K,
    char *form="%4d   %c   %c %4d %4d %4d  %5.1f %5.1f  %5.1f %5.1f  %6.2f %6.1f %4.2f   %3s\n";
    TYPE betinv[2], *bet=beta;
 #endif
-   int nreps, incA, incB, incC, inc, nmat, k;
+   #ifdef ATL_USETHREADS
+      const size_t cs=ATL_NTHREADS *
+                      ATL_MulBySize(ATL_DivBySize(CACHESIZE+ATL_sizeof-1));
+   #else
+      const size_t cs=ATL_MulBySize(ATL_DivBySize(CACHESIZE+ATL_sizeof-1));
+   #endif
+   const size_t setsz = M*N + M*K + N*K;
+   size_t nset;
+   size_t i, incA, incB, incC, inc, k;
    TYPE *c, *C, *a, *A, *b, *B, *st;
-   int ii, jj, i, j=0, PASSED, nerrs;
+   int nreps, ii, jj, j=0, PASSED, nerrs;
    double t0, t1, t2, t3, mflop, mf, mops;
    TYPE maxval, f1, ferr;
    static TYPE feps=0.0;
    static int itst=1;
    enum ATLAS_TRANS TAc, TBc;
    void *vp;
-
    #ifdef TCPLX
       if (*beta == 0.0 && beta[1] == 0.0) betinv[0] = betinv[1] = 0.0;
       else if (beta[1] == 0.0) { betinv[0] = 1 / *beta;  betinv[1] = 0.0; }
@@ -382,17 +550,21 @@ int mmcase0(int MFLOP, int CACHESIZE, char TA, char TB, int M, int N, int K,
       else TBc = AtlasTrans;
    }
    incC = ldc*N;
-   inc = incA + incB + incC;
-   i = M*K + K*N + M*N;  /* amount of inc actually referenced */
-   /* This is a hack; change to use of flushcache instead. */
-   nmat = ((CACHESIZE/ATL_sizeof) + i)/i;
-   vp = malloc(ATL_MulBySize(nmat*inc)+ATL_Cachelen);
+   inc = incA + incB + incC;   /* amount actually allocated */
+   nset = (cs+setsz-1)/setsz;
+   if (nset < 2)
+      nset = 2;
+   vp = malloc(ATL_MulBySize(nset*inc)+ATL_Cachelen);
    ATL_assert(vp);
    C = c = ATL_AlignPtr(vp);
    a = A = C + incC;
    b = B = A + incA;
-   st = C + nmat*inc;
-   matgen(inc, nmat, C, inc, M*N);
+   st = C + nset*inc;
+   #ifdef ATL_NANC
+      matnan(inc, nset, C, inc, M*N);
+   #else
+      matgen(inc, nset, C, inc, M*N);
+   #endif
 
 #ifdef DEBUG
    printmat("A0", M, K, A, lda);
@@ -423,7 +595,7 @@ int mmcase0(int MFLOP, int CACHESIZE, char TA, char TB, int M, int N, int K,
    printmat("C", M, N, C, ldc);
 #endif
 
-   matgen(inc, nmat, C, inc, M*N);
+   matgen(inc, nset, C, inc, M*N);
    t0 = time00();
    for (k=nreps; k; k--)
    {
@@ -446,7 +618,17 @@ int mmcase0(int MFLOP, int CACHESIZE, char TA, char TB, int M, int N, int K,
    if (t1 == t2) t3 = 1.0;
    else if (t2 != 0.0) t3 = t1/t2;
    else t3 = 0.0;
-   printf(form, itst++, TA, TB, M, N, K, MALPH, MBETA, t2, mflop, t3, pc);
+   #ifdef ATL_RKXOVER
+      if (K <= ATL_vMAX_KB)
+      {
+         if (SCALAR_IS_ZERO(beta))
+            speedup_rk_b0[K] = (t3 >= 1.0) ? t3*100 : 0;
+         else
+            speedup_rk[K] = (t3 >= 1.0) ? t3*100 : 0;
+      }
+   #endif
+   printf(form, itst-4, TA, TB, M, N, K, MALPH, MBETA, t2, mflop, t3, pc);
+   itst++;
    free(vp);
    return(1);
 }
@@ -464,14 +646,16 @@ void GetFlags(int nargs, char *args[], int *TEST, int *nside,
               enum ATLAS_DIAG **Diag, int *M0, int *MN, int *Minc,
               int *N0, int *NN, int *Ninc, int *K0, int *KN, int *Kinc,
               int *nalphas, TYPE **alphas, int *nbetas, TYPE **betas,
-              int *LDA_IS_M, int *MFLOP, int *CACHESIZE)
+              int *LDA_IS_M, int *MFLOP, int *CACHESIZE, int *nrep)
 
 {
    char ch;
    int i, j;
+   char *outfile=NULL;
 /*
  * Set up defaults
  */
+   *nrep = 1;
    *TEST = 1;
    *M0 = *N0 = *K0 = -1;
    *nuplo = *nta = *ntb = *nside = *ndiag = *nalphas = *nbetas = -1;
@@ -486,6 +670,9 @@ void GetFlags(int nargs, char *args[], int *TEST, int *nside,
    {
       switch(args[i][1])
       {
+      case '#':
+         *nrep = atoi(args[++i]);
+         break;
       case 'F':
          *MFLOP = atoi(args[++i]);
          break;
@@ -597,6 +784,11 @@ void GetFlags(int nargs, char *args[], int *TEST, int *nside,
       case 'd':
          *LDA_IS_M  = atoi(args[++i]);
          break;
+      #ifdef ATL_RKXOVER
+      case 'o':
+         outfile = args[++i];
+         break;
+      #endif
       default:
          PrintUsage(args[0]);
          break;
@@ -607,10 +799,42 @@ void GetFlags(int nargs, char *args[], int *TEST, int *nside,
  */
    if (*N0 == -1)
    {
-      *N0 = 100;
-      *NN = 1000;
-      *Ninc = 100;
+      #ifdef ATL_RKXOVER
+         #ifdef SCPLX
+            *N0 = *NN = *Ninc = 3000;
+         #elif defined(DCPLX)
+            *N0 = *NN = *Ninc = 2000;
+         #elif defined(SREAL)
+            *N0 = *NN = *Ninc = 4000;
+         #else
+            *N0 = *NN = *Ninc = 3000;
+         #endif
+      #else
+         *N0 = 100;
+         *NN = 1000;
+         *Ninc = 100;
+      #endif
    }
+#ifdef ATL_RKXOVER
+   *K0 = 1;
+   *KN = ATL_vMAX_KB;
+   *Kinc = 1;
+   if (!outfile)
+      outfile = Mstr(Mjoin(Mjoin(atlas_,PRE),rkXover.h));
+   fprk = fopen(outfile, "w");
+   assert(fprk);
+   *nbetas  = 2;
+   *betas  = malloc(ATL_MulBySize(1));
+   assert(*betas );
+   #ifdef TREAL
+      **betas  = 1.0;
+      (*betas)[1] = 0.0;
+   #else
+      **betas  = 1.0;
+      (*betas)[2] = (*betas)[3] = (*betas)[1] = 0.0;
+   #endif
+   *TEST = 0;
+#endif
    if (*nside == -1)
    {
       *nside = 1;
@@ -669,13 +893,17 @@ void GetFlags(int nargs, char *args[], int *TEST, int *nside,
          **betas  = 1.0;
          (*betas)[1] = 0.0;
       #endif
+      #ifdef ATL_NANC
+         **betas = 0.0;
+      #endif
    }
 }
-___main(){}
-__main(){}
-MAIN__(){}
-_MAIN_(){}
-main(int nargs, char *args[])
+int ___main(){return(-1);}   /* this nonsense helps */
+int __main(){return(-1);}    /* when you link with certain */
+int MAIN__(){return(-1);}    /* fortran compilers */
+int _MAIN_(){return(-1);}
+
+int main(int nargs, char *args[])
 /*
  *  tst <tst> <# TA> <TA's> <# TB's> <TB's> <M0> <MN> <incM> <N0> <NN> <incN>
  *      <K0> <KN> <incK> <# alphas> <alphas> <# betas> <betas>
@@ -684,8 +912,8 @@ main(int nargs, char *args[])
 {
    int M0, MN, incM, N0, NN, incN, K0, KN, incK, lda, ldb, ldc, MFLOP;
    int i, k, m, n, im, in, ik, ita, itb, ia, ib, nTA, nTB, nalph, nbeta;
-   int itst=0, ipass=0, TEST, LDA_IS_M, MSAME=0, KSAME=0;
-   int ndiag, nuplo, nside;
+   int itst=0, ipass=0, TEST, LDA_IS_M, MSAME=0, KSAME=0, r;
+   int ndiag, nuplo, nside, nrep;
    TYPE *alph, *beta, *A, *B, *C, *D=NULL;
    #ifdef TREAL
       TYPE bet1 = 1.0, alp1 = -1.0;
@@ -702,7 +930,7 @@ main(int nargs, char *args[])
    GetFlags(nargs, args, &TEST, &nside, &Side, &nuplo, &Uplo,
             &nTA, &TransA, &nTB, &TransB, &ndiag, &Diag,
             &M0, &MN, &incM, &N0, &NN, &incN, &K0, &KN, &incK,
-            &nalph, &alph, &nbeta, &beta, &LDA_IS_M, &MFLOP,&CACHESIZE);
+            &nalph, &alph, &nbeta, &beta, &LDA_IS_M, &MFLOP,&CACHESIZE, &nrep);
 
    if (M0 == -1)
    {
@@ -733,10 +961,15 @@ main(int nargs, char *args[])
  */
    if (MFLOP)
    {
-      mmcase0(10, 1, 'n', 'n', 100, 100, 100, alp1, 100, 100, bet1, 100);
-      mmcase0(10, 1, 'n', 't', 100, 100, 100, alp1, 100, 100, bet1, 100);
-      mmcase0(10, 1, 't', 'n', 100, 100, 100, alp1, 100, 100, bet1, 100);
-      mmcase0(10, 1, 't', 't', 100, 100, 100, alp1, 100, 100, bet1, 100);
+      #ifdef ATL_RKXOVER
+         k = Mmin(100, KN);
+      #else
+         k = 100;
+      #endif
+      mmcase0(10, 1, 'n', 'n', 100, 100, k, alp1, 100, 100, bet1, 100);
+      mmcase0(10, 1, 'n', 't', 100, 100, k, alp1, 100, 100, bet1, 100);
+      mmcase0(10, 1, 't', 'n', 100, 100, k, alp1, 100, 100, bet1, 100);
+      mmcase0(10, 1, 't', 't', 100, 100, k, alp1, 100, 100, bet1, 100);
    }
    else
    {
@@ -756,7 +989,7 @@ main(int nargs, char *args[])
    printf("\nTEST  TA  TB    M    N    K  alpha   beta    Time  Mflop  SpUp  PASS\n");
    printf("====  ==  ==  ===  ===  ===  =====  =====  ======  =====  ====  ====\n\n");
 #else
-   printf("\nTEST  TA  TB    M    N    K        alpha         beta    Time  Mflop  SpUp  PASS\n");
+   printf("\nTEST  TA  TB    M    N    K  ralph ialph  rbeta ibeta    Time  Mflop  SpUp  PASS\n");
    printf("====  ==  ==  ===  ===  ===  ===== =====  ===== =====  ======  =====  ====  ====\n\n");
 #endif
    for (im=M0; im <= MN; im += incM)
@@ -801,31 +1034,53 @@ main(int nargs, char *args[])
                            else ldb = NN;
                            ldc = MN;
                         }
-
-                        if (MFLOP)
+                        for (r=0; r < nrep; r++)
                         {
-                           ipass++;
+                           if (MFLOP)
+                           {
+                              ipass++;
 #ifdef TREAL
-                              mmcase0(MFLOP, CACHESIZE, TA, TB, m, n, k,
-				      alph[ia], lda, ldb, beta[ib], ldc);
+                                 mmcase0(MFLOP, CACHESIZE, TA, TB, m, n, k,
+				         alph[ia], lda, ldb, beta[ib], ldc);
 #else
-                              mmcase0(MFLOP, CACHESIZE, TA, TB, m, n, k,
-				      alph+(ia SHIFT), lda, ldb,
-				      beta+(ib SHIFT), ldc);
+                                 mmcase0(MFLOP, CACHESIZE, TA, TB, m, n, k,
+				         alph+(ia SHIFT), lda, ldb,
+				         beta+(ib SHIFT), ldc);
 #endif
-                        }
-                        else
-                        {
+                           }
+                           else
+                           {
 #ifdef TREAL
-                              ipass += mmcase(TEST, CACHESIZE, TA, TB, m,
-					      n, k, alph[ia], A, lda, B, ldb,
-                                              beta[ib], C, ldc, D,ldc);
+                                 ipass += mmcase(TEST, CACHESIZE, TA, TB, m,
+					         n, k, alph[ia], A, lda, B, ldb,
+                                                 beta[ib], C, ldc, D,ldc);
 #else
-                              ipass += mmcase(TEST, CACHESIZE, TA, TB, m,
-					      n, k, alph+(ia SHIFT), A,
-					      lda, B, ldb, beta+(ib SHIFT),
-					      C, ldc, D,ldc);
+                                 ipass += mmcase(TEST, CACHESIZE, TA, TB, m,
+					         n, k, alph+(ia SHIFT), A,
+					         lda, B, ldb, beta+(ib SHIFT),
+					         C, ldc, D,ldc);
 #endif
+                           }
+/*
+ *                         If we lose 5 times in a row, quit search
+ */
+                           #ifdef ATL_RKXOVER
+                              if (k > 16 &&    /* don't quit too early */
+                                  !(speedup_rk_b0[k] | speedup_rk_b0[k-1] |
+                                    speedup_rk_b0[k-2] | speedup_rk_b0[k-3] |
+                                    speedup_rk_b0[k-4] | speedup_rk[k] |
+                                    speedup_rk_b0[k-1] | speedup_rk_b0[k-2] |
+                                    speedup_rk_b0[k-3] | speedup_rk[k-4] |
+                                    speedup_rk_b0[k-6] | speedup_rk[k-7] |
+                                    speedup_rk_b0[k-8] | speedup_rk[k-9] |
+                                    speedup_rk_b0[k-10] | speedup_rk[k-11]
+                                   )
+                                 )
+                              {
+                                 MAX_vKB = k-5;
+                                 goto RANK_K;
+                              }
+                              #endif
                         }
                      }
                   }
@@ -834,6 +1089,34 @@ main(int nargs, char *args[])
          }
       }
    }
+   #ifdef ATL_RKXOVER
+RANK_K:
+      fprintf(fprk, "#ifndef ATLAS_RKXOVER\n   #define ATLAS_RKXOVER\n\n");
+
+/*
+ *    First, find last size where we got speedup.  Then, if KB > 80, demand
+ *    at > 3% speedup before declaring it worth the extra code
+ */
+      for (speedup_rk_b0[0]=0, n=MAX_vKB; !speedup_rk_b0[n]; n--);
+      while(n > 80 && speedup_rk_b0[n] < 104) n--;
+      fprintf(fprk, "#define ATL_RK_MAXK_B0 %d\n", n);
+      fprintf(fprk, "static int ATL_rkXover_b0[%d] = {0", n+1);
+      for (i=1; i <= n; i++)
+         fprintf(fprk, ",%d", speedup_rk_b0[i]);
+      fprintf(fprk, "};\n");
+
+      for (speedup_rk[0]=0, m=MAX_vKB; !speedup_rk[m]; m--);
+      while(m > 80 && speedup_rk[m] < 104) m--;
+      fprintf(fprk, "#define ATL_RK_MAXK_BX %d\n", m);
+      fprintf(fprk, "static int ATL_rkXover_bX[%d] = {0", m+1);
+      for (i=1; i <= m; i++)
+         fprintf(fprk, ",%d", speedup_rk[i]);
+      fprintf(fprk, "};\n");
+      fprintf(fprk, "#define ATL_RK_MAXK %d\n", m >= n ? m : n);
+
+      fprintf(fprk, "\n#endif\n");
+      fclose(fprk);
+   #endif
    if (TEST && !MFLOP)
       printf("\nNTEST=%d, NUMBER PASSED=%d, NUMBER FAILURES=%d\n",
              itst, ipass, itst-ipass);
@@ -852,5 +1135,5 @@ main(int nargs, char *args[])
       free(C);
       if (D) free(D);
    }
-   exit(0);
+   return(0);
 }
