@@ -1,5 +1,5 @@
 /*
- *             Automatically Tuned Linear Algebra Software v3.8.4
+ *             Automatically Tuned Linear Algebra Software v3.9.51
  *                    (C) Copyright 2000 R. Clint Whaley
  *
  * Redistribution and use in source and binary forms, with or without
@@ -27,7 +27,6 @@
  * POSSIBILITY OF SUCH DAMAGE.
  *
  */
-
 #include "atlas_misc.h"
 #include "atlas_lapack.h"
 #include "cblas.h"
@@ -52,8 +51,13 @@ double time00();
       ATL_assert(Mjoin(PATL,f77gels)(TA_, M_, N_, NRHS_, A_, lda_, B_, ldb_)==0)
 #else
    #include "clapack.h"
-   #define Cposv Mjoin(Mjoin(clapack_,PRE),posv)
+   #ifdef ATL_FULL_LAPACK
+      #include "C_lapack.h"
+   #endif
    #define Cgesv Mjoin(Mjoin(clapack_,PRE),gesv)
+   #define Cposv Mjoin(Mjoin(clapack_,PRE),posv)
+   #define Cgels Mjoin(Mjoin(clapack_,PRE),gels)
+//   #define Cgels Mjoin(PATL,gels)
    #define test_posv(Ord_, Uplo_, N_, NRHS_, A_, lda_, B_, ldb_) \
    { \
       int ii; \
@@ -64,7 +68,7 @@ double time00();
    #define test_gesv(Ord_, N_, NRHS_, A_, lda_, ipiv_, B_, ldb_) \
       ATL_assert(Cgesv(Ord_, N_, NRHS_, A_, lda_, ipiv_, B_, ldb_) == 0)
    #define test_gels(Ord_, TA_, M_, N_, NRHS_, A_, lda_, B_, ldb_) \
-      exit(-1);  /* no C iface to gels */
+       ATL_assert(!Cgels(Ord_,TA_, M_, N_, NRHS_, A_, lda_, B_, ldb_));
 #endif
 
 void PrintUsage(char *nam)
@@ -257,7 +261,7 @@ static void PosDefGen
  * Generates a reasonably conditioned positive definite matrix
  */
 {
-   TYPE *aa, t0, *L;
+   TYPE *aa, *L;
    TYPE val, bias, sign;
    int j;
    const int lda2=(lda SHIFT), ldap1=((lda+1)SHIFT);
@@ -326,7 +330,6 @@ static void MakeHEDiagDom
  * Makes hermitian matrix diagonally dominant
  */
 {
-   TYPE *aa, t0;
    int j;
    const int lda2=(lda SHIFT), ldap1=((lda+1)SHIFT);
 
@@ -446,21 +449,13 @@ static TYPE slvtst(int N, int NRHS, TYPE nrmA, TYPE *XC, int ldxc,
    return(max);
 }
 
-static TYPE geslvtst(enum CBLAS_ORDER Order, int N, int NRHS, TYPE *A, int lda,
+static TYPE geslvtst(enum CBLAS_ORDER Order, int N, int NRHS, TYPE nrmA,
                      TYPE *XC, int ldxc, TYPE *X, int ldx, double thresh)
 /*
  * Figures ||x - xc|| / (N*eps*||A||*||xc||), where xc is known correct
  * answer, and x is computed result
  */
 {
-   TYPE nrmA;
-/*
- * We don't account for Order, so this isn't the real norm, but it should
- * still work OK -- it's essentially the norm of A'
- * NOTE: I think this is an error, because I'm actually passing LU or QR
- *       in A, and I want original A.  Need to change!
- */
-   nrmA = Mjoin(PATL,genrm1)(N, N, A, lda);
    return(slvtst(N, NRHS, nrmA, XC, ldxc, X, ldx, thresh));
 }
 
@@ -604,6 +599,7 @@ static int gelstst(enum CBLAS_ORDER Order, int N, int NRHS, int lda, int ldb,
  * Generate A and X, and figure B with matmul
  */
    A = GetGE(N, N, lda);
+   nrm = Mjoin(PATL,genrm1)(N, N, A, lda);
    X = GetGE(N, NRHS, ldb);
    B = GetGE(N, NRHS, ldb);
    ATL_assert(A && X && B);
@@ -624,7 +620,7 @@ static int gelstst(enum CBLAS_ORDER Order, int N, int NRHS, int lda, int ldb,
    if (F == NULL || t1 <= 0.0) t1 = mflop = -1.0;
    else mflop = mflop / (t1*1000000.0);
 
-   nrm = geslvtst(Order, N, NRHS, A, lda, X, ldb, B, ldb, thresh);
+   nrm = geslvtst(Order, N, NRHS, nrm, X, ldb, B, ldb, thresh);
    if (nrm > thresh) ierr=1;
 
    fprintf(stdout, "  %c     %c %7d %7d %7d %7d %10.3f %9.2f  %e\n",
@@ -662,6 +658,7 @@ static int gesvtst(enum CBLAS_ORDER Order, int N, int NRHS, int lda, int ldb,
  * Generate A and X, and figure B with matmul
  */
    A = GetGE(N, N, lda);
+   nrm = Mjoin(PATL,genrm1)(N, N, A, lda);
    X = GetGE(N, NRHS, ldb);
    B = GetGE(N, NRHS, ldb);
    ipiv = malloc(N*sizeof(int));
@@ -683,11 +680,12 @@ static int gesvtst(enum CBLAS_ORDER Order, int N, int NRHS, int lda, int ldb,
    if (F == NULL || t1 <= 0.0) t1 = mflop = -1.0;
    else mflop = mflop / (t1*1000000.0);
 
-   nrm = geslvtst(Order, N, NRHS, A, lda, X, ldb, B, ldb, thresh);
+   nrm = geslvtst(Order, N, NRHS, nrm, X, ldb, B, ldb, thresh);
    if (nrm > thresh) ierr=1;
 
    fprintf(stdout, "  %c     %c %7d %7d %7d %7d %10.3f %9.2f  %e\n",
            cord, 'G', N, NRHS, lda, ldb, t1, mflop, nrm);
+   free(ipiv);
    free(A);
    free(X);
    free(B);
