@@ -1,5 +1,5 @@
 /*
- *             Automatically Tuned Linear Algebra Software v3.8.4
+ *             Automatically Tuned Linear Algebra Software v3.9.51
  *                    (C) Copyright 1997 R. Clint Whaley
  *
  * Redistribution and use in source and binary forms, with or without
@@ -126,7 +126,11 @@
 #define CPAT Mjoin(C_ATL_, PRE);
 
 #ifndef ATL_MaxMalloc
-   #define ATL_MaxMalloc 67108864
+   #ifdef ATL_MaxMalloc_MB
+      #define ATL_MaxMalloc (((size_t)(ATL_MaxMalloc_MB))<<20)
+   #else
+      #define ATL_MaxMalloc 67108864
+   #endif
 #endif
 
 typedef void (*MAT2BLK)(int, int, const TYPE*, int, TYPE*, const SCALAR);
@@ -201,6 +205,11 @@ void Mjoin(PATL,IBJBmm)(int IB, int JB, int K, const TYPE *A, const TYPE *B,
                         const TYPE beta, TYPE *C, const int ldc);
 void Mjoin(PATL,IBNBmm)(const int M, const int K, const TYPE *A, const TYPE *B,
                         const TYPE beta, TYPE *C, const int ldc);
+void Mjoin(PATL,putblk_bX)(int M, int N, TYPE *V, TYPE *C, int ldc, const SCALAR beta);
+void Mjoin(PATL,putblk_bn1)(int M, int N, TYPE *V, TYPE *C, int ldc, const SCALAR beta);
+void Mjoin(PATL,putblk_b1)(int M, int N, TYPE *V, TYPE *C, int ldc, const SCALAR beta);
+void Mjoin(PATL,putblk_b0)(int M, int N, TYPE *V, TYPE *C, int ldc, const SCALAR beta);
+
 #ifdef TCPLX
 
 void Mjoin(PATL,CNBmm_b0)(const int M, const int N, const int K,
@@ -215,6 +224,7 @@ void Mjoin(PATL,CNBmm_bX)(const int M, const int N, const int K,
                           const TYPE alpha, const TYPE *A, const int lda,
                           const TYPE *B, const int ldb, const TYPE beta,
                           TYPE *C, const int ldc);
+void Mjoin(PATL,putblk_bXi0)(int M, int N, TYPE *V, TYPE *C, int ldc, const SCALAR beta);
 void Mjoin(PATL, gescal_bXi0)(const int, const int, const SCALAR, TYPE*,
            const int);
 
@@ -263,15 +273,11 @@ void Mjoin(PATL,mmIJK2)
     const SCALAR alpha, const TYPE *A, const int lda, TYPE *pA0, const int incA,
     MAT2BLK A2blk, TYPE *pB0, const SCALAR beta, TYPE *C, int ldc,
     MATSCAL gescal, NBMM0 NBmm0);
+void ATL_gereal2cplx(const int M, const int N, TYPE *alpha, TYPE *R, int ldr,
+                     TYPE *I, int ldi, TYPE *beta, TYPE *C, int ldc);
 
 #else  /* real */
 
-void Mjoin(PATL,putblk_bX)(int M, int N, TYPE *V, TYPE *C, int ldc, const SCALAR beta);
-void Mjoin(PATL,putblk_bn1)(int M, int N, TYPE *V, TYPE *C, int ldc, const SCALAR beta);
-void Mjoin(PATL,putblk_b1)(int M, int N, TYPE *V, TYPE *C, int ldc, const SCALAR beta);
-void Mjoin(PATL,putblk_b0)(int M, int N, TYPE *V, TYPE *C, int ldc, const SCALAR beta);
-void ATL_gereal2cplx(const int M, const int N, TYPE *alpha, TYPE *R, int ldr,
-                     TYPE *I, int ldi, TYPE *beta, TYPE *C, int ldc);
 
 void NBmm_b1(const int M, const int N, const int K, const TYPE alpha,
              const TYPE *A, const int lda, const TYPE *B, const int ldb,
@@ -359,6 +365,12 @@ void Mjoin(PATL,row2blkT_a1)(int, int, const TYPE*, int, TYPE*, const SCALAR);
 void Mjoin(PATL,col2blk2_a1)(int, int, const TYPE*, int, TYPE*, const SCALAR);
 void Mjoin(PATL,col2blk_a1)(int, int, const TYPE*, int, TYPE*, const SCALAR);
 
+#ifdef TREAL
+int Mjoin(PATL,vrankK)(const enum ATLAS_TRANS TA, const enum ATLAS_TRANS TB,
+                       ATL_CINT M, ATL_CINT N, ATL_CINT K, const SCALAR alpha,
+                       const TYPE *A, ATL_CINT lda, const TYPE *B,
+                       ATL_CINT ldb, const SCALAR beta, TYPE *C, ATL_CINT ldc);
+#endif
 int Mjoin(PATL,mmJITcp)(const enum ATLAS_TRANS TA, const enum ATLAS_TRANS TB,
                         const int M, const int N, const int K,
                         const SCALAR alpha, const TYPE *A, const int lda,
@@ -426,6 +438,48 @@ void Mjoin(PATL,aliased_gemmNN)
    (const int M, const int N, const int K, const SCALAR alpha,
     const TYPE *A, const int lda, const TYPE *B, const int ldb,
     const SCALAR beta, TYPE *C, const int ldc);
+#ifdef TCPLX
+void Mjoin(PATL,gemmCC)
+   (const int M, const int N, const int K, const SCALAR alpha,
+    const TYPE *A, const int lda, const TYPE *B, const int ldb,
+    const SCALAR beta, TYPE *C, const int ldc);
+void Mjoin(PATL,aliased_gemmCC)
+   (const int M, const int N, const int K, const SCALAR alpha,
+    const TYPE *A, const int lda, const TYPE *B, const int ldb,
+    const SCALAR beta, TYPE *C, const int ldc);
+void Mjoin(PATL,gemmCT)
+   (const int M, const int N, const int K, const SCALAR alpha,
+    const TYPE *A, const int lda, const TYPE *B, const int ldb,
+    const SCALAR beta, TYPE *C, const int ldc);
+void Mjoin(PATL,aliased_gemmCT)
+   (const int M, const int N, const int K, const SCALAR alpha,
+    const TYPE *A, const int lda, const TYPE *B, const int ldb,
+    const SCALAR beta, TYPE *C, const int ldc);
+void Mjoin(PATL,gemmCN)
+   (const int M, const int N, const int K, const SCALAR alpha,
+    const TYPE *A, const int lda, const TYPE *B, const int ldb,
+    const SCALAR beta, TYPE *C, const int ldc);
+void Mjoin(PATL,aliased_gemmCN)
+   (const int M, const int N, const int K, const SCALAR alpha,
+    const TYPE *A, const int lda, const TYPE *B, const int ldb,
+    const SCALAR beta, TYPE *C, const int ldc);
+void Mjoin(PATL,gemmTC)
+   (const int M, const int N, const int K, const SCALAR alpha,
+    const TYPE *A, const int lda, const TYPE *B, const int ldb,
+    const SCALAR beta, TYPE *C, const int ldc);
+void Mjoin(PATL,aliased_gemmTC)
+   (const int M, const int N, const int K, const SCALAR alpha,
+    const TYPE *A, const int lda, const TYPE *B, const int ldb,
+    const SCALAR beta, TYPE *C, const int ldc);
+void Mjoin(PATL,gemmNC)
+   (const int M, const int N, const int K, const SCALAR alpha,
+    const TYPE *A, const int lda, const TYPE *B, const int ldb,
+    const SCALAR beta, TYPE *C, const int ldc);
+void Mjoin(PATL,aliased_gemmNC)
+   (const int M, const int N, const int K, const SCALAR alpha,
+    const TYPE *A, const int lda, const TYPE *B, const int ldb,
+    const SCALAR beta, TYPE *C, const int ldc);
+#endif
 
 
 void NCmmNNIJK_c
