@@ -1,5 +1,5 @@
 /*
- *             Automatically Tuned Linear Algebra Software v3.8.4
+ *             Automatically Tuned Linear Algebra Software v3.9.51
  *                    (C) Copyright 2003 R. Clint Whaley
  *
  * Redistribution and use in source and binary forms, with or without
@@ -395,6 +395,12 @@ void ATL_cpcol2blk_a1_blk
 void ATL_cprow2blkT_a1_blk
    (const int blk, const int M, const int N, const float* alpha,
     const float *A, int lda, const int ldainc, float *V);
+void ATL_cprow2blkTF_blk
+   (const int blk, const int M, const int N, const float* alpha,
+    const float *A, int lda, const int ldainc, float *V);
+void ATL_cpcol2blkF_blk
+   (const int blk, const int M, const int N, const float* alpha,
+    const float *A, int lda, const int ldainc, float *V);
 void ATL_cpcol2blkConjF
    (const int M, const int N, const float* alpha,
     const float *A, int lda, const int ldainc, float *V);
@@ -486,6 +492,12 @@ void ATL_zpcol2blk_a1_blk
 void ATL_zprow2blkT_a1_blk
    (const int blk, const int M, const int N, const double* alpha,
     const double *A, int lda, const int ldainc, double *V);
+void ATL_zprow2blkTF_blk
+   (const int blk, const int M, const int N, const double* alpha,
+    const double *A, int lda, const int ldainc, double *V);
+void ATL_zpcol2blkF_blk
+   (const int blk, const int M, const int N, const double* alpha,
+    const double *A, int lda, const int ldainc, double *V);
 void ATL_zpcol2blkConjF
    (const int M, const int N, const double* alpha,
     const double *A, int lda, const int ldainc, double *V);
