@@ -1,5 +1,5 @@
 /*
- *             Automatically Tuned Linear Algebra Software v3.8.4
+ *             Automatically Tuned Linear Algebra Software v3.9.51
  *                    (C) Copyright 1999 R. Clint Whaley
  *
  * Redistribution and use in source and binary forms, with or without
@@ -38,10 +38,7 @@
 
 double cblas_dznrm2(const int N, const void *X, const int incX)
 {
-   if (N > 0)
-   {
-      if (incX < 0) return(ATL_dznrm2(N, X, -incX));
-      else return(ATL_dznrm2(N, X, incX));
-   }
-   else return(0.0);
+   if (N > 0 && incX > 0)
+      return(ATL_dznrm2(N, X, incX));
+   return(0.0);
 }
