@@ -1,5 +1,5 @@
 /*
- *             Automatically Tuned Linear Algebra Software v3.8.4
+ *             Automatically Tuned Linear Algebra Software v3.9.51
  *                    (C) Copyright 1999 R. Clint Whaley
  *
  * Redistribution and use in source and binary forms, with or without
@@ -87,15 +87,11 @@ void cblas_sgbmv(const enum CBLAS_ORDER Order, const enum CBLAS_TRANSPOSE TA,
       if (incY < 0) y += (1-N)*incY;
    }
    if (Order == CblasColMajor)
-   {
-      if (TA == CblasNoTrans)
-         ATL_sgbmv(TA, M, N, KL, KU, alpha, A, lda, x, incX, beta, y, incY);
-      else ATL_sgbmv(TA, N, M, KL, KU, alpha, A, lda, x, incX, beta, y, incY);
-   }
+      ATL_sgbmv(TA, M, N, KL, KU, alpha, A, lda, x, incX, beta, y, incY);
    else
    {
       if (TA == CblasNoTrans)
-         ATL_sgbmv(CblasTrans, M, N, KU, KL, alpha, A, lda, x, incX,
+         ATL_sgbmv(CblasTrans, N, M, KU, KL, alpha, A, lda, x, incX,
                    beta, y, incY);
       else ATL_sgbmv(CblasNoTrans, N, M, KU, KL, alpha, A, lda, x, incX,
                      beta, y, incY);
