@@ -1,5 +1,5 @@
 /*
- *             Automatically Tuned Linear Algebra Software v3.8.4
+ *             Automatically Tuned Linear Algebra Software v3.9.51
  *                    (C) Copyright 1999 R. Clint Whaley
  *
  * Redistribution and use in source and binary forms, with or without
@@ -27,9 +27,11 @@
  * POSSIBILITY OF SUCH DAMAGE.
  *
  */
-
 #define DREAL
 #include "atlas_misc.h"
+#ifdef ATL_USEPTHREADS
+   #include "atlas_ptalias_lapack.h"
+#endif
 #include "atlas_lapack.h"
 #include "clapack.h"
 
