@@ -1,5 +1,5 @@
 /*
- *             Automatically Tuned Linear Algebra Software v3.8.4
+ *             Automatically Tuned Linear Algebra Software v3.9.51
  *                   (C) Copyright 2001 Peter Soendergaard
  *
  * Code contributers : Peter Soendergaard, R. Clint Whaley
@@ -31,6 +31,9 @@
  */
 #define SREAL
 #include "atlas_misc.h"
+#ifdef ATL_USEPTHREADS
+   #include "atlas_ptalias_lapack.h"
+#endif
 #include "atlas_lapack.h"
 #include "clapack.h"
 
