@@ -1,6 +1,6 @@
 /*
- *             Automatically Tuned Linear Algebra Software v3.8.4
- *                    (C) Copyright 1999 R. Clint Whaley
+ *             Automatically Tuned Linear Algebra Software v3.9.51
+ * Copyright (C) 2009, 1999 R. Clint Whaley
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -30,12 +30,12 @@
 #include "atlas_misc.h"
 
 
-static void axpy_lt8(const int N, const SCALAR alpha, const TYPE *x, TYPE *y)
+static void axpy_lt8(ATL_CINT N, const SCALAR alpha, const TYPE *x, TYPE *y)
 /*
  * For cleanup, see if we can get compiler to do the work, use constant loops
  */
 {
-   int i;
+   ATL_INT i;
    const register TYPE ralpha = *alpha, ialpha = alpha[1];
    register TYPE xr, xi;
 
@@ -140,9 +140,9 @@ static void axpy_lt8(const int N, const SCALAR alpha, const TYPE *x, TYPE *y)
    #define PEQ +=
    #define MEQ -=
 #endif
-static void axpy_8(const int N, const SCALAR alpha, const TYPE *x, TYPE *y)
+static void axpy_8(ATL_CINT N, const SCALAR alpha, const TYPE *x, TYPE *y)
 {
-   const int n4 = N >> 2, N2 = (n4>>1)<<1, nr = n4 - N2, nn4 = n4<<1;
+   ATL_CINT n4 = N >> 2, N2 = (n4>>1)<<1, nr = n4 - N2, nn4 = n4<<1;
    TYPE *u = y+nn4, *v = u+nn4, *z = v+nn4;
    const TYPE *X1 = x + nn4, *X2 = X1 + nn4, *X3 = X2 + nn4;
    const TYPE *stX = x + ((N2-2)<<1);
@@ -317,7 +317,7 @@ static void axpy_8(const int N, const SCALAR alpha, const TYPE *x, TYPE *y)
    #undef PEQ
    #undef MEQ
 #elif defined(ATL_NOMULADD) && ATL_mmnreg >= 26
-static void axpy_8(const int N, const SCALAR alpha, const TYPE *x, TYPE *y)
+static void axpy_8(ATL_CINT N, const SCALAR alpha, const TYPE *x, TYPE *y)
 /*
  * 8 register prefetch on X & Y, with 4 cycle multiply & 4 cycle add,
  * unrolled by 16 to ensure multiple cacheline usage for both singe & double
@@ -506,11 +506,11 @@ void Mjoin(PATL,axpy_x1_y1)
  * y <- alpha * x + y
  */
 #endif
-   (const int N, const SCALAR alpha, const TYPE *X, const int incX,
-    TYPE *Y, const int incY)
+   (ATL_CINT N, const SCALAR alpha, const TYPE *X, ATL_CINT incX,
+    TYPE *Y, ATL_CINT incY)
 {
 #if defined(ATL_NOMULADD) && ATL_mmnreg >= 26
-   const int n8 = (N>>3)<<3, nr = N - n8;
+   ATL_CINT n8 = (N>>3)<<3, nr = N - n8;
 
    if (n8)
    {
