@@ -1,6 +1,6 @@
 /*
- *             Automatically Tuned Linear Algebra Software v3.8.4
- *                    (C) Copyright 1999 R. Clint Whaley
+ *             Automatically Tuned Linear Algebra Software v3.9.51
+ * Copyright (C) 2009, 1999 R. Clint Whaley
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -31,7 +31,7 @@
 
 
 void Mjoin(Mjoin(PATL,gescal),BNM)
-   (const int M, const int N, const SCALAR beta0, TYPE *C, const int ldc)
+   (ATL_CINT M, ATL_CINT N, const SCALAR beta0, TYPE *C, ATL_CINT ldc)
 /*
  * C <- beta * C
  */
@@ -40,7 +40,7 @@ void Mjoin(Mjoin(PATL,gescal),BNM)
    Mjoin(PATL,gezero)(M, N, C, ldc);
 #elif defined (TREAL)
 
-   const int incC = (ldc << 1), incC2 = incC - M, incC3 = ldc - M;
+   ATL_CINT incC = (ldc << 1), incC2 = incC - M, incC3 = ldc - M;
    TYPE *C0 = C, *C1 = C0 + ldc;
    TYPE *stM = C0 + M, *stN = C0 + ldc*N;
    register TYPE beta=beta0;
@@ -100,11 +100,11 @@ void Mjoin(Mjoin(PATL,gescal),BNM)
 #if defined(BETAXI0)
    Mjoin(ATL_,Mjoin(UPR,gescal_bX))(M<<1, N, *beta0, C, ldc<<1);
 #else
-   const int incC=(ldc<<2)-(M<<1), n=N>>1;
+   ATL_CINT incC=(ldc<<2)-(M<<1), n=N>>1;
    const register TYPE rbeta = *beta0, ibeta = beta0[1];
    TYPE *C0=C, *C1 = C + (ldc<<1);
    register TYPE r0, r1, i0, i1, ar0, ai0, ar1, ai1;
-   register int j, i;
+   register ATL_INT j, i;
 
    for (j=n; j; j--, C0 += incC, C1 += incC)
    {
