@@ -1,5 +1,5 @@
 /*
- *             Automatically Tuned Linear Algebra Software v3.8.4
+ *             Automatically Tuned Linear Algebra Software v3.9.51
  *                    (C) Copyright 1997 R. Clint Whaley
  *
  * Redistribution and use in source and binary forms, with or without
@@ -822,7 +822,7 @@ static void trsmcpUN
    int j;
 
    tmp[0] = *alpha; tmp[1] = alpha[1];
-   Mjoin(PATL,cplxinvert)(1, tmp, 2, tmp, 2); /* safe cplx inversion */
+   Mjoin(PATL,cplxinvert)(1, tmp, 1, tmp, 1); /* safe cplx inversion */
 /*
  * copy non-diagonal portion of matrix, scaled by 1/alpha
  */
@@ -853,7 +853,7 @@ static void trsmcpUT
    else move = Mjoin(PATL,cpsc);
 
    tmp[0] = *alpha; tmp[1] = alpha[1];
-   Mjoin(PATL,cplxinvert)(1, tmp, 2, tmp, 2); /* safe cplx inversion */
+   Mjoin(PATL,cplxinvert)(1, tmp, 1, tmp, 1); /* safe cplx inversion */
 /*
  * copy non-diagonal portion of matrix, scaled by 1/alpha
  */
@@ -883,7 +883,7 @@ static void trsmcpLT(const enum ATLAS_TRANS Trans, const int N,
    else move = Mjoin(PATL,cpsc);
 
    tmp[0] = *alpha; tmp[1] = alpha[1];
-   Mjoin(PATL,cplxinvert)(1, tmp, 2, tmp, 2); /* safe cplx inversion */
+   Mjoin(PATL,cplxinvert)(1, tmp, 1, tmp, 1); /* safe cplx inversion */
 /*
  * copy non-diagonal portion of matrix, scaled by 1/alpha
  */
@@ -907,7 +907,7 @@ static void trsmcpLN(const int N, const TYPE *alpha, const TYPE *A,
    int j;
 
    tmp[0] = *alpha; tmp[1] = alpha[1];
-   Mjoin(PATL,cplxinvert)(1, tmp, 2, tmp, 2); /* safe cplx inversion */
+   Mjoin(PATL,cplxinvert)(1, tmp, 1, tmp, 1); /* safe cplx inversion */
 /*
  * copy non-diagonal portion of matrix, scaled by 1/alpha
  */
@@ -947,7 +947,7 @@ static enum ATLAS_DIAG trsmcopy
    {
       if (Trans != AtlasConjTrans) Mjoin(PATL,copy)(N, A, lda+1, C, N+1);
       else Mjoin(PATL,copyConj)(N, A, lda+1, C, N+1);
-      Mjoin(PATL,cplxinvert)(N, C, N+N+2, C, N+N+2); /* safe cplx inversion */
+      Mjoin(PATL,cplxinvert)(N, C, N+1, C, N+1); /* safe cplx inversion */
       Mjoin(PATL,scal)(N, alpha, C, N+1);
    }
    return(diag);
@@ -1009,7 +1009,7 @@ void Mjoin(PATL,CtrsmKL)
             break;
          default: /* this crap should never be used */
                tmp[0] = ATL_rone; tmp[1] = ATL_rzero;
-               Mjoin(PATL,cplxinvert)(M, a, M+M+2, a, M+M+2);
+               Mjoin(PATL,cplxinvert)(M, a, M+1, a, M+1);
                Mjoin(PATL,reftrsm)(AtlasLeft, Uplo, AtlasNoTrans, Diag, M, N,
                                    tmp, a, M, B, ldb);
          }
@@ -1028,7 +1028,7 @@ void Mjoin(PATL,CtrsmKL)
             tmp[0] = A[0];
             if (Trans != AtlasConjTrans) tmp[1] = A[1];
             else tmp[1] = -A[1];
-            Mjoin(PATL,cplxinvert)(1, tmp, 2, tmp, 2); /* safe cplx invers */
+            Mjoin(PATL,cplxinvert)(1, tmp, 1, tmp, 1); /* safe cplx invers */
             ra = tmp[0]; ia = tmp[1];
             tmp[0] = *alpha * ra - alpha[1] * ia;
             tmp[1] = *alpha * ia + alpha[1] * ra;
