@@ -1,5 +1,5 @@
 /*
- *             Automatically Tuned Linear Algebra Software v3.8.4
+ *             Automatically Tuned Linear Algebra Software v3.9.51
  *                    (C) Copyright 1999 R. Clint Whaley
  *
  * Redistribution and use in source and binary forms, with or without
@@ -32,11 +32,26 @@
 #include "atlas_level3.h"
 #include "atlas_level1.h"
 #include "atlas_lapack.h"
+#include "atlas_lamch.h"
+#include Mstr(Mjoin(Mjoin(atlas_,PRE),sysinfo.h))
+#ifdef ATL_USEPTHREADS
+   #include "atlas_pthreads.h"
+   #include "atlas_tcacheedge.h"
+#else
+   #include "atlas_cacheedge.h"
+#endif
+
 
 #ifdef TREAL
    #define ATL_luMmin 2
+   #define ATL_PCAMin 1500
 #else
    #define ATL_luMmin 1
+   #ifdef SCPLX
+      #define ATL_PCAMin 256
+   #else
+      #define ATL_PCAMin 512
+   #endif
 #endif
 
 #ifdef TCPLX
@@ -53,7 +68,7 @@ int ATL_getrfR(const int M, const int N, TYPE *A, const int lda, int *ipiv)
  */
 {
    const int MN = Mmin(M, N);
-   int Nup, Ndown, i, ierr=0;
+   int Nup, Ndown, i, k, ierr=0;
    #ifdef TCPLX
       const TYPE one[2] = {ATL_rone, ATL_rzero};
       const TYPE none[2] = {ATL_rnone, ATL_rzero};
@@ -96,7 +111,13 @@ int ATL_getrfR(const int M, const int N, TYPE *A, const int lda, int *ipiv)
          tmp = A[i];
          if (tmp != ATL_rzero)
          {
-            cblas_scal(N, ATL_rone/tmp, A, 1);
+            if (Mabs(tmp) >= ATL_laSAFMIN)
+               cblas_scal(N, ATL_rone/tmp, A, 1);
+            else
+            {
+               for (k=0; k < N; k++)
+                  A[k] /= tmp;
+            }
             A[i] = *A;
             *A = tmp;
          }
@@ -107,8 +128,13 @@ int ATL_getrfR(const int M, const int N, TYPE *A, const int lda, int *ipiv)
          tmp[1] = A[i+1];
          if (tmp[0] != ATL_rzero || tmp[1] != ATL_rzero)
          {
-            ATL_CplxInv(tmp, inv);
-            cblas_scal(N, inv, A, 1);
+            if (ATL_lapy2(tmp[0], tmp[1]) >= ATL_laSAFMIN)
+            {
+               ATL_CplxInv(tmp, inv);
+               cblas_scal(N, inv, A, 1);
+            }
+            else
+               Mjoin(PATL,cplxdivide)(N, tmp, A, 1, A, 1);
             A[i] = *A;
             A[i+1] = A[1];
             *A = tmp[0];
