@@ -4,7 +4,7 @@
  * -- Automatically Tuned Linear Algebra Software (ATLAS)
  *    (C) Copyright 2000 All Rights Reserved
  *
- * -- ATLAS routine -- Version 3.2 -- December 25, 2000
+ * -- ATLAS routine -- Version 3.9.24 -- December 25, 2000
  *
  * Author         : Antoine P. Petitet
  * Originally developed at the University of Tennessee,
@@ -344,7 +344,7 @@ void Mjoin( PATL, pther2k )
                                      LDC );
    ATL_join_tree  ( root );
    ATL_free_tree  ( root );
-   ATL_thread_exit( &attr );
+   ATL_thread_free( &attr );
 /*
  * End of Mjoin( PATL, pther2k )
  */
