@@ -1,5 +1,5 @@
 /*
- *             Automatically Tuned Linear Algebra Software v3.8.4
+ *             Automatically Tuned Linear Algebra Software v3.9.51
  *                    (C) Copyright 2006 R. Clint Whaley
  *
  * Redistribution and use in source and binary forms, with or without
@@ -1537,7 +1537,10 @@ ALIGN8
         fstpl   CMUL(8)(pC0)
         fstpl   (pC0)
         fstpl   CMUL(16)(pC0)
-#elif defined(BETA1)
+#elif defined(BETA1) || defined(BETAN1)
+   #ifdef BETAN1
+      #define faddl fsubl
+   #endif
                                  /* st = rC3, rC1, rC0, rC2  */
         fxch    %st(2)           /* st = rC0, rC1, rC3, rC2 */
         faddl   (pC0)            /* st = rC0+,rC1, rC3, rC2 */
@@ -1549,6 +1552,9 @@ ALIGN8
         fstpl   CMUL(16)(pC0)    /* st = rC3 */
         faddl   CMUL(24)(pC0)    /* st = rC3+ */
         fstpl   CMUL(24)(pC0)    /* st = null */
+   #ifdef BETAN1
+      #undef  faddl
+   #endif
 #else
                                  /* st = rC3, rC1, rC0, rC2  */
         fldl    BETAOFF(%rsp)    /* st = bet, rC3, rC1, rC0, rC2 */
