@@ -1,5 +1,5 @@
 /*
- *             Automatically Tuned Linear Algebra Software v3.8.4
+ *             Automatically Tuned Linear Algebra Software v3.9.51
  *                    (C) Copyright 2008 R. Clint Whaley
  *
  * Redistribution and use in source and binary forms, with or without
@@ -30,6 +30,7 @@
 #include "atlas_asm.h"
 
 
+#include "atlas_asm.h"   /* usually do this in include file */
 #if !defined(MB)
    #define MB 0
 #endif
@@ -81,7 +82,6 @@
    #define ldab         %edx
    #define pfA          %ebp
    #define stM          %ebx
-   #define stN          STNOFF(%esp)
 
    #define subL subl
    #define addL addl
