@@ -1,4 +1,4 @@
-!              Automatically Tuned Linear Algebra Software v3.8.4
+!              Automatically Tuned Linear Algebra Software v3.9.51
 !                     (C) Copyright 2002 R. Clint Whaley
 !
 !  Redistribution and use in source and binary forms, with or without
@@ -121,6 +121,12 @@
 !                const TYPE *A, const int lda, const TYPE *B, const int ldb,
 !                      [%fp+100] [%fp+104]     [%fp+108]
 !                const TYPE beta, TYPE *C, const int ldc)
+#ifdef ATL_USE64BITS
+        .register       %g2, #scratch
+        .register       %g3, #scratch
+        .register       %g6, #scratch
+#endif
+
 	.section	".text"
 	.align	8
 	.global	ATL_USERMM
