@@ -1,5 +1,5 @@
 /*
- *             Automatically Tuned Linear Algebra Software v3.8.4
+ *             Automatically Tuned Linear Algebra Software v3.9.51
  *                    (C) Copyright 1997 R. Clint Whaley
  *
  * Redistribution and use in source and binary forms, with or without
@@ -260,8 +260,15 @@
 #ifdef TREAL
    #define NBmm Mjoin(Mjoin(Mjoin(Mjoin(Mjoin(ppre,MNKnam), TRnam),ldnam), ALPHAnam), BETAnam)
 
+#ifdef NEWMM
+   #ifndef ATL_CINT
+      #define ATL_CINT const int
+   #endif
+   void NBmm(ATL_CINT M, ATL_CINT N, ATL_CINT K, TYPE *A, TYPE *B, TYPE *C);
+#else
    void NBmm(const int, const int, const int, const SCALAR, const TYPE*,
              const int, const TYPE*, const int, const SCALAR, TYPE*, const int);
+#endif
 #else
    #define NBmm0 Mjoin(Mjoin(Mjoin(Mjoin(ppre,MNKnam), TRnam),ldnam), ALPHAnam)
    #define _bn1 _bX
@@ -605,6 +612,12 @@ int mmtst(void)
                     i, j, C0[k], C1[k]);
             nerr++;
          }
+         else if (C1[k] != C1[k])   /* test for NaNs in test answer */
+         {
+            fprintf(stderr, "C(%d,%d) : expected=%f, got=%f\n",
+                    i, j, C0[k], C1[k]);
+            nerr++;
+         }
       }
    }
    free(vc);
@@ -613,11 +626,11 @@ int mmtst(void)
    return(nerr);
 }
 
-main()
+int main()
 {
    int ierr;
    ierr = mmtst();
    if (!ierr) fprintf(stdout, "PASSED TEST\n");
-   exit(ierr);
+   return(ierr);
 }
 
