@@ -1,5 +1,5 @@
 /*
- *             Automatically Tuned Linear Algebra Software v3.8.4
+ *             Automatically Tuned Linear Algebra Software v3.9.51
  *                    (C) Copyright 1997 R. Clint Whaley
  *
  * Redistribution and use in source and binary forms, with or without
@@ -43,17 +43,17 @@
 #endif
 #define dumb_rand() ( 0.5 - ((double)rand())/((double)RAND_MAX) )
 
+
+int ___main(){return(-1);}   /* this nonsense helps */
+int __main(){return(-1);}    /* when you link with certain */
+int MAIN__(){return(-1);}    /* fortran compilers */
+int _MAIN_(){return(-1);}
+
+double time00();
 #define nshape 5
 enum ATLAS_MATSHAPE {AtlasM_NB=0, AtlasN_NB=1, AtlasMN_NB=2, AtlasK_NB=3,
                      Atlas0_NB=4, AtlasMN_REST};
 
-___main(){}
-__main(){}
-MAIN__(){}
-_MAIN_(){}
-
-double time00();
-
 #define DENMAT 175
 #define MAXALLOC (8*1024*1024*8)
 
@@ -63,35 +63,35 @@ F77_INTEGER ATL_Slen1, ATL_Slen2;
 
 void Mjoin(PATL,small_mm)
    (const enum ATLAS_TRANS TA, const enum ATLAS_TRANS TB,
-    const int M, const int N, const int K, const SCALAR alpha,
-    const TYPE *A, const int lda, const TYPE *B, const int ldb,
-    const SCALAR beta, TYPE *C, const int ldc);
+    ATL_CINT M, ATL_CINT N, ATL_CINT K, const SCALAR alpha,
+    const TYPE *A, ATL_CINT lda, const TYPE *B, ATL_CINT ldb,
+    const SCALAR beta, TYPE *C, ATL_CINT ldc);
 void Mjoin(PATL,big_mm)
    (const enum ATLAS_TRANS TA, const enum ATLAS_TRANS TB,
-    const int M, const int N, const int K, const SCALAR alpha,
-    const TYPE *A, const int lda, const TYPE *B, const int ldb,
-    const SCALAR beta, TYPE *C, const int ldc);
+    ATL_CINT M, ATL_CINT N, ATL_CINT K, const SCALAR alpha,
+    const TYPE *A, ATL_CINT lda, const TYPE *B, ATL_CINT ldb,
+    const SCALAR beta, TYPE *C, ATL_CINT ldc);
 void Mjoin(PATL,dbig_mm)
    (const enum ATLAS_TRANS TA, const enum ATLAS_TRANS TB,
-    const int M, const int N, const int K, const SCALAR alpha,
-    const TYPE *A, const int lda, const TYPE *B, const int ldb,
-    const SCALAR beta, TYPE *C, const int ldc);
+    ATL_CINT M, ATL_CINT N, ATL_CINT K, const SCALAR alpha,
+    const TYPE *A, ATL_CINT lda, const TYPE *B, ATL_CINT ldb,
+    const SCALAR beta, TYPE *C, ATL_CINT ldc);
 void Mjoin(PATL,sbig_mm)
    (const enum ATLAS_TRANS TA, const enum ATLAS_TRANS TB,
-    const int M, const int N, const int K, const SCALAR alpha,
-    const TYPE *A, const int lda, const TYPE *B, const int ldb,
-    const SCALAR beta, TYPE *C, const int ldc);
+    ATL_CINT M, ATL_CINT N, ATL_CINT K, const SCALAR alpha,
+    const TYPE *A, ATL_CINT lda, const TYPE *B, ATL_CINT ldb,
+    const SCALAR beta, TYPE *C, ATL_CINT ldc);
 typedef void (*GEMMPTR)
    (const enum ATLAS_TRANS TA, const enum ATLAS_TRANS TB,
-    const int M, const int N, const int K, const SCALAR alpha,
-    const TYPE *A, const int lda, const TYPE *B, const int ldb,
-    const SCALAR beta, TYPE *C, const int ldc);
+    ATL_CINT M, ATL_CINT N, ATL_CINT K, const SCALAR alpha,
+    const TYPE *A, ATL_CINT lda, const TYPE *B, ATL_CINT ldb,
+    const SCALAR beta, TYPE *C, ATL_CINT ldc);
 static GEMMPTR big_gemm, small_gemm;
 
-void matgen(int M0, int N, TYPE *A, int lda0, int seed)
+void matgen(ATL_INT M0, ATL_INT N, TYPE *A, ATL_INT lda0, int seed)
 {
-   int i, j;
-   const int M = M0 SHIFT, lda = lda0 SHIFT;
+   ATL_INT i, j;
+   ATL_CINT M = M0 SHIFT, lda = lda0 SHIFT;
 
    dumb_seed(seed);
    for (j=N; j; j--)
@@ -495,7 +495,7 @@ void DoShapes(FILE *fpout, char TA, char TB, int nb, int N0, int NN, int incN,
    }
 }
 
-main(int nargs, char *args[])
+int main(int nargs, char *args[])
 {
    char TA='n', TB='n';
    int nb, N0, NN, incN, jstop=0;
@@ -548,5 +548,5 @@ main(int nargs, char *args[])
    fprintf(fp, "\n#endif\n");
    fclose(fp);
 #endif
-   exit(0);
+   return(0);
 }
