@@ -1,6 +1,7 @@
 /*
- *             Automatically Tuned Linear Algebra Software v3.8.4
- *                    (C) Copyright 2000 R. Clint Whaley
+ *             Automatically Tuned Linear Algebra Software v3.9.51
+ * Copyright (C) 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
+ *               R. Clint Whaley
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -35,6 +36,8 @@
 #include "atlas_misc.h"
 #include "atlas_fopen.h"
 #include "atlas_prefetch.h"
+#include "atlas_gentesttime.h"
+#include "atlas_mmtesttime.h"
 
 #define Mmin(x, y) ( (x) > (y) ? (y) : (x) )
 
@@ -632,6 +635,8 @@ MULTHEAD *BuildTable(char pre, enum CLEAN_WHICH which, int nb)
       io1 = 0;
       io2 = 1;
       break;
+   case CleanNot:
+      exit(-1);
    }
 
    n = NumUserCases(pre);
@@ -702,24 +707,11 @@ int GetPNB(char pre, enum CLEAN_WHICH which, int icase, int NB, int imul,
 #define NO_RESULTS -88.7
 double GetRes(char *fnam)
 {
-   FILE *fp;
-   double mf, mflop[3];
-   int i;
-
-   fp = fopen(fnam, "r");
-   if (!fp) return(NO_RESULTS);
-   for (i=0; i != 3; i++)
-   {
-      if (fscanf(fp, " %lf", &mflop[i]) != 1)
-      {
-         fclose(fp);
-         remove(fnam);
-         return(NO_RESULTS);
-      }
-   }
-   fclose(fp);
-   mf = GetAvg(3, TOLERANCE, mflop);
-   return(mf);
+   double *dp;
+   dp = ReadResultsFile(0, 3, fnam);
+   if (dp)
+      return(*dp);
+   return(NO_RESULTS);
 }
 double ummcase0
 (
@@ -754,8 +746,7 @@ double ummcase0
       i += sprintf(ln+i, "pre=%c muladd=%d lat=%d M=%d N=%d K=%d mb=%d nb=%d kb=%d mu=%d nu=%d ku=%d lda=%d ldb=%d ldc=%d ",
                    pre, muladd, lat, M, N, K, mb, nb, kb, mu, nu, ku,
                    lda, ldb, ldc);
-      i += sprintf(ln+i, "\n");
-      fprintf(stdout, "%s", ln);
+      i += sprintf(ln+i, "> /dev/null 2>&1\n");
       if (system(ln) != 0) return(-1.0);
    }
    mf = GetRes(outnam);
@@ -1019,9 +1010,13 @@ int ummtstcase0
    i += sprintf(ln+i, "pre=%c muladd=%d lat=%d M=%d N=%d K=%d mb=%d nb=%d kb=%d mu=%d nu=%d ku=%d lda=%d ldb=%d ldc=%d ",
                 pre, muladd, lat, M, N, K, mb, nb, kb, mu, nu, ku,
                 lda, ldb, ldc);
-   i += sprintf(ln+i, "\n");
-   fprintf(stdout, "%s", ln);
-   return(system(ln) == 0);
+   i += sprintf(ln+i, "> /dev/null 2>&1\n");
+   fprintf(stdout, "     TESTING PRE='%c' FILE='%s', NB=%d . . .",
+           pre, fnam, nb);
+   i = system(ln);
+   if (!i) fprintf(stdout, " PASSED!\n");
+   else fprintf(stdout, " FAILED!\n");
+   return(i == 0);
 }
 
 static int Mylcm(const int M, const int N)
@@ -1083,12 +1078,13 @@ MinIsOdd:
    else return(M);
 }
 
-int GetUserNB(char pre, int NB, int mb, int nb, int kb)
+int GetUserNB(char pre, int NB, int mb, int nb, int kb, int iflag)
 /*
  * Given ATLAS's preference for NB, finds a good nb based on user's input
  */
 {
-   int i, nelt, mult;
+   int i, mult;
+   static int nelt=0;
 
    if (mb < 0)
    {
@@ -1101,13 +1097,25 @@ int GetUserNB(char pre, int NB, int mb, int nb, int kb)
       else return(-nb);
    }
    else if (kb < 0) return(-kb);
+   if (!nelt)
+      nelt = L1Elts(pre, 128*1024);
    if (!nb) nb = 1;
    if (!mb) mb = 1;
    if (!kb) kb = 1;
    mult = Mylcm(mb,nb);
    mult = Mylcm(kb,mult);
+   if (iflag&512)  /* needs to keep 4 matrices in cache */
+   {
+      if ( (NB%mb == 0) && (NB%nb == 0) && (NB%kb == 0) && 3*NB*NB <= nelt)
+         return(NB);
+      i = (NB/mult)*mult;
+      if (4*i*i <= nelt)
+         return(i);
+      for (; i >= 16; i -= mult)
+         if (4*i*i <= nelt || i <= 16)
+            return(i);
+   }
    if ( (NB%mb == 0) && (NB%nb == 0) && (NB%kb == 0) ) return(NB);
-   nelt = L1Elts(pre, 128*1024);
    i = ((NB+mult-1)/mult)*mult;
    if (i*i < nelt) return(i);
    else return((NB/mult)*mult);
@@ -1156,46 +1164,71 @@ int FindBestUser(char pre, int nb0)
 {
    char *MCC, *MMFLAGS;
    char ln[256], fnam[256];
-   double mf, mfbest=0.0;
+   double mf, mfbest=0.0, adv;
    int ibest=(-1), i, ncases;
-   int ID, iflag, NB, mb, nb, kb, ma, lat, mu, nu, ku;
+   int ID, iflag, NB, mb, nb, kb, ma, lat, mu, nu, ku, nbB=0;
+   FILE *fpsum;
 
+   sprintf(ln, "res/%cucases.res", pre);
+   fpsum = fopen(ln, "w");
    ncases = NumUserCases(pre);
    for (i=0; i < ncases; i++)
    {
       ID = GetUserCase(pre, -i, &iflag, &mb, &nb, &kb, &ma, &lat,
                        &mu, &nu, &ku, fnam, ln, &MCC, &MMFLAGS);
       assert(ID > 0);
-      NB = GetUserNB(pre, nb0, mb, nb, kb);
+      NB = GetUserNB(pre, nb0, mb, nb, kb, iflag);
       if (NB)
       {
          mf = ummcase(pre, ID, NB);
-         if (mf > mfbest)
+         if (NB > 80 && nbB < NB-8)
+            adv = 1.08;           /* penalize large NB due to cleanup probs */
+         else if (NB > nbB)       /* slightly favor small NB */
+            adv = 1.001;
+         else if (NB < nbB)       /* slightly favor small NB */
+            adv = 0.999;
+         else
+            adv = 1.0;
+
+         if (mf > adv*mfbest)
          {
             if (utstmmcase(pre, ID, NB))
             { /* test kernel before accepting */
                ibest = ID;
                mfbest = mf;
+               nbB = NB;
             }
          }
          fprintf(stdout, "%3d. NB=%3d, rout=%40s, MFLOP=%.2f\n",
                  i, NB, fnam, mf);
+         if (fpsum)
+            fprintf(fpsum, "%3d. ID=%8d, NB=%3d, rout=%40s, MFLOP=%.2f\n",
+                    i, ID, NB, fnam, mf);
       }
    }
+   if (fpsum)
+      fclose(fpsum);
+   printf("BEST CASE IS ID=%d, NB=%d, MFLOP=%.2f\n", ibest, nbB, mfbest);
    return(ibest);
 }
 
 void PrintUsage(char *fnam)
 {
-   fprintf(stderr, "\nUSAGE: %s -p <pre> -n <nb>\n\n", fnam);
+   fprintf(stderr,
+           "\nUSAGE: %s -p <pre> -n <nb> -e <search summary file\n\n", fnam);
    exit(-1);
 }
 
-void GetFlags(int nargs, char **args, char *pre, int *nb0,
-              enum CLEAN_WHICH *which)
+ROUTNODE *GetFlags(int nargs, char **args, char *pre, int *nb0,
+                   enum CLEAN_WHICH *which)
+/*
+ * RETURNS: a queue of filenames, which are the output files from external
+ *          searches (using atlas_mmtesttime.h format).
+ */
 {
-   int i;
+   int i, ID=10000;
    char ch;
+   ROUTNODE *efb=NULL, *ef, *ep;/* base &ptr to external search summary files */
 
    *nb0 = 0;
    *pre = 'd';
@@ -1214,6 +1247,17 @@ void GetFlags(int nargs, char **args, char *pre, int *nb0,
       case 'n':
          *nb0 = atoi(args[++i]);
          break;
+      case 'e':
+         if (++i > nargs)
+            PrintUsage(args[0]);
+          ep = CreateRoutNode(args[i], ID, 0.0);
+          if (efb)
+             ef->next = ep;
+          else
+             efb = ep;
+          ef = ep;
+          ID += 5000;
+          break;
       case 'C':
          switch(args[++i][0])
          {
@@ -1234,6 +1278,7 @@ void GetFlags(int nargs, char **args, char *pre, int *nb0,
          PrintUsage(args[0]);
       }
    }
+   return(efb);
 }
 
 int FindBestNB(pre, icase)
@@ -1399,6 +1444,7 @@ void RunTimes(char pre)
    FILE *fp;
    int j, icase, nb;
    char ln[128];
+   ATL_mmnode_t *mmp;
 
    sprintf(ln, "res/%cuMMRES", pre);
    if (FileExists(ln))
@@ -1417,9 +1463,12 @@ void RunTimes(char pre)
    }
    else
    {
-      sprintf(ln, "res/%cMMRES", pre);
-      if (!FileExists(ln)) sprintf(ln, "res/%cgMMRES", pre);
-      GetInstLogFile(ln, pre, &j, &j, &j, &nb, &j, &j, &j, &j, &j, &j, &mf);
+      mmp = ReadMMFileWithPath(pre, "res", "MMRES.sum");
+      if (!mmp)
+         mmp = ReadMMFileWithPath(pre, "res", "gMMRES.sum");
+      assert(mmp);
+      nb = (mmp->next) ? mmp->next->nbB : mmp->nbB;
+      KillAllMMNodes(mmp);
       FindUMM(pre, nb);
    }
 }
@@ -1430,6 +1479,7 @@ void GetuMMRES(char pre)
    FILE *fp;
    int j, icase, nb;
    char ln[128];
+   ATL_mmnode_t *mmp;
 
    sprintf(ln, "res/%cuMMRES", pre);
    if (FileExists(ln))
@@ -1448,22 +1498,195 @@ void GetuMMRES(char pre)
    }
    else
    {
-      sprintf(ln, "res/%cMMRES", pre);
-      if (!FileExists(ln)) sprintf(ln, "res/%cgMMRES", pre);
-      GetInstLogFile(ln, pre, &j, &j, &j, &nb, &j, &j, &j, &j, &j, &j, &mf);
+      mmp = ReadMMFileWithPath(pre, "res", "MMRES.sum");
+      if (!mmp)
+         mmp = ReadMMFileWithPath(pre, "res", "gMMRES.sum");
+      assert(mmp);
+      nb = (mmp->next) ? mmp->next->nbB : mmp->nbB;
+      KillAllMMNodes(mmp);
       FindUMM(pre, nb);
    }
 }
 
-main(int nargs, char **args)
+ATL_mmnode_t *ReadOldUserFile(char pre)
+/*
+ * Reads original res/<pre>uMMRES file, and returns filled-in ATL_mmnode_t
+ */
 {
-   int i, nb0;
-   enum CLEAN_WHICH which;
-   char pre;
+   char ln[1024], ln2[1024];
+   double mf;
+   int id, nb, iflag, i;
+   ATL_mmnode_t *mmp;
+   FILE *fp;
 
-   GetFlags(nargs, args, &pre, &nb0, &which);
-   if (nb0 <= 0) GetuMMRES(pre);
-   else if (which == CleanNot) FindUMM(pre, nb0);
-   else FindUClean(pre, nb0, which);
-   exit(0);
+   mmp = GetMMNode();
+   sprintf(ln, "res/%cuMMRES", pre);
+   fp = fopen(ln, "r");
+   assert(fp);
+   assert(fgets(ln, 1024, fp));
+   assert(fgets(ln, 1024, fp));
+   sscanf(ln, "%d %d %lf", &id, &nb, &mf);
+   fclose(fp);
+   mmp->ID = id;
+   mmp->mflop[0] = mf;
+   mmp->mbB = mmp->nbB = mmp->kbB = nb;
+   assert(GetUserCase(pre, id, &iflag, &i, &i, &i, &mmp->muladd, &mmp->lat,
+                      &mmp->mu, &mmp->nu, &mmp->ku, ln, ln2,
+                      &mmp->comp, &mmp->cflags));
+   mmp->comp = DupString(mmp->comp);
+   mmp->cflags = DupString(mmp->cflags);
+   i = strlen(ln) + 7;
+   mmp->rout= malloc(i * sizeof(char));
+   assert(mmp->rout);
+   mmp->auth = DupString(ln2);
+   sprintf(mmp->rout, "CASES/%s", ln);
+   if (iflag & 32)
+   {
+      SET_FLAG(mmp->flag, MMF_LDISKB, 0);
+      SET_FLAG(mmp->flag, MMF_LDAB, 0);
+   }
+   if (iflag & 64)
+      SET_FLAG(mmp->flag, MMF_MRUNTIME, 1);
+   if (iflag & 128)
+      SET_FLAG(mmp->flag, MMF_NRUNTIME, 1);
+   if (iflag & 256)
+      SET_FLAG(mmp->flag, MMF_KRUNTIME, 1);
+   return(mmp);
+}
+
+void getBestOfSearches
+(
+   char pre,            /* precision prefix */
+   int verb,            /* verbosity */
+   ROUTNODE *esumb      /* list of search output files */
+)
+/*
+ * Scopes all external search summary files, output res/<pre>eMMRES.sum
+ * file describing best user/external kernel available.
+ */
+{
+   ATL_mmnode_t *mmb, *mmp, *mmB;
+   double mfB;
+   ROUTNODE *ep;
+   char ln[2048];
+
+   mmb = ReadMMFileWithPath(pre, "res", "eMMRES.sum");
+   if (mmb)    /* already have output file from previous run */
+   {
+/*
+ *    Rerun external searches to force generation of any needed files
+ */
+      for (ep=esumb; ep; ep = ep->next)
+      {
+         sprintf(ln, "make %s\n", ep->rout);
+         if (system(ln))
+            fprintf(stderr, "EXTERNAL SEARCH FAILED: %s", ln);
+      }
+/*
+ *    Retime best kernel, and rewrite output file if necessary
+ */
+      if (mmb->mflop[0] <= 0.0)
+      {
+         TimeAllMMKernels(0, verb, 0, mmb, pre, 0, 0, 0, 1, 0, 0);
+         WriteMMFileWithPath(pre, "res", "eMMRES.sum", mmb);
+      }
+/*
+ *    Echo best case to screen and return
+ */
+      fprintf(stdout, "\nBEST OF USER CASES AND EXTERNAL SEARCHES:\n");
+      PrintMMNodes(stdout, mmb);
+      KillAllMMNodes(mmb);
+      return;
+   }
+   mmb = mmp = ReadOldUserFile(pre);   /* get best user case */
+   assert(mmb);
+   for (ep=esumb; ep; ep = ep->next)
+   {
+      sprintf(ln, "make %s\n", ep->rout);
+      if (system(ln))
+      {
+         fprintf(stderr, "EXTERNAL SEARCH FAILED: %s", ln);
+         continue;
+      }
+      mmp->next = ReadMMFile(ep->rout);
+      if (!mmp->next)
+      {
+         fprintf(stderr, "EXTERNAL SEARCH FAILED: %s", ep->rout);
+         continue;
+      }
+/*
+ *    If MFLOPs not set, time and the write back out
+ */
+      if (mmp->next->mflop[0] <= 0.0)
+      {
+         TimeAllMMKernels(0, verb, 1, mmp->next, pre, 0, 0, 0, 1, 0, 0);
+         WriteMMFile(ep->rout, mmp->next);
+      }
+/*
+ *    Right now, should always be one kernel, but if there are multiple,
+ *    they all get considered, so find the last one as link in the queue
+ *    to be compared
+ */
+      while (mmp->next)
+         mmp = mmp->next;
+   }
+/*
+ * Nuke kernels that don't pass basic tests, time survivors, in mflop[1]
+ * NOTE: could time BETA cases separately and build best-of-breed, but
+ *       this would require that kernels have same NB, and be kind of
+ *       messy, so for now assume perf follows BETA=1 case
+ */
+   mmb = DelBadMMKernels(pre, verb, mmb);
+   TimeAllMMKernels(1, verb, 1, mmb, pre, 0, 0, 0, 1, 0, 0);
+/*
+ * Pick fastest kernel whose NB <= 80
+ */
+   mfB = mmb->mflop[1];
+   mmB = mmb;
+   for (mmp=mmb->next; mmp; mmp = mmp->next)
+   {
+      if (mmp->nbB <= 80 && mfB < mmp->mflop[1])
+      {
+         mfB = mmp->mflop[1];
+         mmB = mmp;
+      }
+   }
+   mmB = CloneMMNode(mmB);
+   KillAllMMNodes(mmb);
+   WriteMMFileWithPath(pre, "res", "eMMRES.sum", mmB);
+   fprintf(stdout, "\nBEST OF USER CASES AND EXTERNAL SEARCHES:\n");
+   PrintMMNodes(stdout, mmB);
+   KillAllMMNodes(mmB);
+}
+
+int main(int nargs, char **args)
+{
+   int i, nb0, verb=1;
+   enum CLEAN_WHICH which;
+   char pre, ln[16];
+   ROUTNODE *esumb;
+
+   esumb = GetFlags(nargs, args, &pre, &nb0, &which);
+   if (nb0 <= 0)
+   {
+      GetuMMRES(pre);
+      getBestOfSearches(pre, verb, esumb);
+   }
+   else if (which == CleanNot)
+   {
+      sprintf(ln, "res/%cuMMRES", pre);
+      if (FileExists(ln))
+         GetuMMRES(pre);
+      else
+         FindUMM(pre, nb0);
+      getBestOfSearches(pre, verb, esumb);
+   }
+   else
+   {
+      getBestOfSearches(pre, verb, esumb);
+      FindUClean(pre, nb0, which);
+   }
+   KillAllRoutNodes(esumb);
+   fprintf(stderr, "SUCCESSFUL FINISH FOR %s\n\n", args[0]);
+   return(0);
 }
