@@ -1,6 +1,7 @@
 #include "atlas_misc.h"
 #include <assert.h>
 #include <string.h>
+#include <ctype.h>
 
 #define AlphaX 2
 typedef struct FileNode FILENODE;
@@ -692,7 +693,7 @@ void GenIncCases(char pre, char *spc, char *ifp, FILE *fpout, int alpha,
             fprintf(fpout, "%sreturn;\n", spc);
             return;
          }
-         else fprintf(fpout, "%s%selse\n%s{\n", spc, spc);
+         else fprintf(fpout, "%selse\n%s{\n", spc, spc);
          fprintf(fpout, "%s   %s(N, alpha, X, incx, beta, Y, incy);\n", spc,
                  GetNam(pre, alpha, beta, ix[i], iy[i]));
          fprintf(fpout, "%s   return;\n%s}\n", spc, spc);
@@ -1237,10 +1238,10 @@ char GetFlags(int nargs, char **args)
    }
    return(ch);
 }
-main(int nargs, char **args)
+int main(int nargs, char **args)
 {
    char pre;
    pre = GetFlags(nargs, args);
    GoToTown(pre);
-   exit(0);
+   return(0);
 }
