@@ -1,6 +1,7 @@
 #include "atlas_misc.h"
 #include <assert.h>
 #include <string.h>
+#include <ctype.h>
 
 int CONJ=0; /* elegant global variable to shoehorn in conjugation support */
 
@@ -1057,10 +1058,10 @@ char GetFlags(int nargs, char **args)
    }
    return(ch);
 }
-main(int nargs, char **args)
+int main(int nargs, char **args)
 {
    char pre;
    pre = GetFlags(nargs, args);
    GoToTown(pre);
-   exit(0);
+   return(0);
 }
