@@ -1,6 +1,7 @@
 #include "atlas_misc.h"
 #include <assert.h>
 #include <string.h>
+#include <ctype.h>
 
 #define AlphaX 2
 typedef struct FileNode FILENODE;
@@ -905,10 +906,10 @@ char GetFlags(int nargs, char **args)
    }
    return(ch);
 }
-main(int nargs, char **args)
+int main(int nargs, char **args)
 {
    char pre;
    pre = GetFlags(nargs, args);
    GoToTown(pre);
-   exit(0);
+   return(0);
 }
