@@ -117,9 +117,10 @@ void FA_free(void *ptr, int align, int misalign)
          for (ap=allocQ; ap && ap->memA != ptr; ap = ap->next) prev = ap;
          if (!ap)
          {
-            fprintf(stderr, "Couldn't find mem=%ld\nmemQ=\n", ptr);
+            fprintf(stderr, "Couldn't find mem=%ld\nmemQ=\n", (size_t)ptr);
             for (ap=allocQ; ap; ap = ap->next)
-               fprintf(stderr, "   %ld, %ld\n", ap->memA, ap->mem);
+               fprintf(stderr, "   %ld, %ld\n", (size_t)ap->memA,
+                       (size_t)ap->mem);
          }
          assert(ap);
          if (ap == allocQ)
@@ -681,7 +682,7 @@ void GetFlags(int nargs, char **args, int *nN, int **Ns, int *nX, int **incXs,
       FAy = sizeof(TYPE);
 }
 
-main(int nargs, char **args)
+int main(int nargs, char **args)
 {
    int nN, nX, nY, nal, nbe;
    int *Ns, *incXs, *incYs;
@@ -694,5 +695,5 @@ main(int nargs, char **args)
    if (betas) free(betas);
    free(incXs);
    free(incYs);
-   exit(ierr);
+   return(ierr);
 }
