@@ -1,5 +1,5 @@
 /*
- *             Automatically Tuned Linear Algebra Software v3.8.4
+ *             Automatically Tuned Linear Algebra Software v3.9.51
  *                    (C) Copyright 2001 R. Clint Whaley
  *
  * Redistribution and use in source and binary forms, with or without
@@ -33,6 +33,7 @@
 #include "cblas.h"
 #include "atlas_cblastypealias.h"
 #include "atlas_tst.h"
+#include "atlas_level3.h"
 
 #ifdef TimeC
    #include "clapack.h"
@@ -111,7 +112,7 @@ static void test_inv(const enum CBLAS_ORDER Order, const enum TEST_UPLO Uplo,
                      const int N, TYPE *A, const int lda)
 {
    if (Uplo == TestGE) geinv(Order, N, A, lda);
-   else poinv(Order, (enum CBLAS_ORDER) Uplo, N, A, lda);
+   else poinv(Order, Uplo, N, A, lda);
 }
 
 TYPE *GetGE(int M, int N, int lda)
@@ -184,7 +185,7 @@ static void PosDefGen
  * Generates a reasonably conditioned positive definite matrix
  */
 {
-   TYPE *aa, t0, *L;
+   TYPE *aa, *L;
    TYPE val, bias, sign;
    int j;
    const int lda2=(lda SHIFT), ldap1=((lda+1)SHIFT);
@@ -253,7 +254,6 @@ static void MakeHEDiagDom
  * Makes hermitian matrix diagonally dominant
  */
 {
-   TYPE *aa, t0;
    int j;
    const int lda2=(lda SHIFT), ldap1=((lda+1)SHIFT);
 
@@ -391,7 +391,7 @@ static void ReflectHE
 static TYPE *GetMat(enum CBLAS_ORDER Order, enum TEST_UPLO Uplo, int N, int lda)
 {
    if (Uplo == TestGE) return(GetGE(N, N, lda));
-   return(GetHE(Order, Uplo, N, lda));
+   return(GetHE(Order, (enum CBLAS_UPLO)Uplo, N, lda));
 }
 
 static TYPE geresid(enum CBLAS_ORDER Order, int N, TYPE *A, int lda,
@@ -474,7 +474,7 @@ static TYPE GetResid(enum CBLAS_ORDER Order, enum TEST_UPLO Uplo,
 {
    TYPE ret;
    if (Uplo == TestGE) ret = geresid(Order, N, A, lda, AI, ldi);
-   else ret = poresid(Order, Uplo, N, A, lda, AI, ldi);
+   else ret = poresid(Order, (enum CBLAS_UPLO)Uplo, N, A, lda, AI, ldi);
    return(ret);
 }
 
@@ -526,8 +526,11 @@ static double RunTiming
    if (A)
    {
       if (Uplo == TestGE)
-         for (i=0; i < k; i++) Mjoin(PATL,gegen)(N, N, A+i*incA, lda, N+lda);
-      else for (i=0; i < k; i++) hegen(Order, Uplo, N, A+i*incA, lda);
+         for (i=0; i < k; i++)
+            Mjoin(PATL,gegen)(N, N, A+i*incA, lda, N+lda);
+      else
+         for (i=0; i < k; i++)
+            hegen(Order, (enum CBLAS_UPLO)Uplo, N, A+i*incA, lda);
 
       t0 = time00();
       for (i=nreps; i; i--, a += incA) test_inv(Order, Uplo, N, a, lda);
@@ -638,7 +641,7 @@ void GetFlags(int nargs, char **args, int *MFLOP, int *CacheSize, int *lda,
       *N = 821;
       *lda = 823;
    #endif
-   *Uplo = CblasUpper;
+   *Uplo = TestUpper;
    *Ord = CblasColMajor;
    for (i=1; i < nargs; i++)
    {
@@ -670,8 +673,8 @@ void GetFlags(int nargs, char **args, int *MFLOP, int *CacheSize, int *lda,
          break;
       case 'U':
          ch = *args[++i];
-         if (ch == 'u' || ch == 'U') *Uplo = AtlasUpper;
-         else if (ch == 'l' || ch == 'L') *Uplo = AtlasLower;
+         if (ch == 'u' || ch == 'U') *Uplo = TestUpper;
+         else if (ch == 'l' || ch == 'L') *Uplo = TestLower;
          else *Uplo = TestGE;
          break;
       default:
@@ -680,9 +683,22 @@ void GetFlags(int nargs, char **args, int *MFLOP, int *CacheSize, int *lda,
    }
    if (*N0 == -1)
    {
-      *N0 = 1;
-      *NN = Mjoin(PATL,GetNB)();
-      *incN = 1;
+      #ifdef TREAL
+         *N0 = 16;
+         i = Mjoin(PATL,GetNB)();
+         if ((i>>3)<<3 == i)
+            *incN = 8;
+         else if ((i>>2)<<2 == i)
+            *incN = 4;
+         else if ((i>>1)<<1 == i)
+            *incN = 2;
+         else
+            *incN = 1;
+         *NN = i << 2;
+      #else
+         incN = *N0 = 1;
+         *NN = Mjoin(PATL,GetNB)();
+      #endif
    }
    if (*lda < *N)
       *lda = *N;
